@@ -2,7 +2,6 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.bytecode;
-import java.io.*;
 
 /** The state of a conditional expression or statement. */
 
@@ -22,14 +21,25 @@ public class IfState {
   /** The stack size before the "then" clause. */
   int start_stack_size;
 
+  /** The change in the stack size caused by the then-clause.
+   * Same as then_stacked_types.length, if stack_growth >= 0. */
+  int stack_growth;
+
   /** The types that were pushed by the then-clause. */
   Type[] then_stacked_types;
 
+  boolean andThenSet;
+
   public IfState (CodeAttr code)
+  {
+    this(code, new Label(code));
+  }
+
+  public IfState (CodeAttr code, Label endLabel)
   {
     previous = code.if_stack;
     code.if_stack = this;
-    end_label = new Label(code);
+    end_label = endLabel;
     start_stack_size = code.SP;
   }
 }
