@@ -9,7 +9,9 @@ public class TryState {
   /** The surrounding TryState, if any. */
   TryState previous;
 
-  /** The label for the code following the entire try-statement. */
+  /** The label for the code following the entire try-statement.
+   * Allocated lazily: If null, the following code is unreachable.
+   */
   Label end_label;
 
   /** If this "try" has a "finally", the Label of the "finally" sub-routine. */
@@ -21,7 +23,7 @@ public class TryState {
   /** Non-null if we need a temporary to save the result. */
   Variable saved_result;
 
-  /** If the SP > 0 when we entered the try, the stack is saved here. */
+  /** If the {@code SP > 0} when we entered the try, the stack is saved here. */
   Variable[] savedStack;
 
   Label start_try;
@@ -33,6 +35,12 @@ public class TryState {
   /** Only used in emitWithCleanupStart mode. */
   Type[] savedTypes;
 
+  ExitableBlock exitCases;
+
+  Variable exception;
+
+  boolean tryClauseDone;
+
   public TryState (CodeAttr code)
   {
     previous = code.try_stack;
@@ -40,4 +48,13 @@ public class TryState {
     start_try = code.getLabel();
   }
 
+  /* Skip TryStates without finally blocks, or if we're no longer in
+   * the try-clause. */
+  static TryState outerHandler (TryState innerTry, TryState outerTry)
+  {
+    while (innerTry != outerTry
+           && (innerTry.finally_subr == null || innerTry.tryClauseDone))
+      innerTry = innerTry.previous;
+    return innerTry;
+  }
 }
