@@ -1,10 +1,18 @@
-// Copyright (c) 2003, 2004, 2006  Per M.A. Bothner.
+// Copyright (c) 2003, 2004, 2006, 2014  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
+
 import gnu.bytecode.*;
 import gnu.mapping.*;
+import gnu.kawa.functions.MakeSplice;
+import gnu.kawa.reflect.CompileArrays;
+import gnu.kawa.io.OutPort;
 import gnu.text.SourceMessages;
+import gnu.text.SyntaxException;
+/* #ifdef use:java.lang.invoke */
+import java.lang.invoke.*;
+/* #endif */
 
 /** This class is used to represent "combination" or "application".
  * A function and arguments are evaluated, and then the function applied.
@@ -26,10 +34,23 @@ public class ApplyExp extends Expression
   /** The next ApplyExp in ((ReferenceExp)func).binding.firstCall list. */
   public ApplyExp nextCall;
 
+    /** Index of argument for first keyword argument.
+     * If zero, no keyword arguments.  If non-zero, then
+     * {@code firstKeywordArgIndex-1} is the index in the {@code args}
+     * array of the first keyword.
+     */
+    public int firstKeywordArgIndex;
+    public int numKeywordArgs;
+
+    /** Index of first argument that is a MakeSplice.
+     * The value is -1 is no argument is a splice.
+     */
+    public int firstSpliceArg = -1;
+
   public final Expression getFunction() { return func; }
   public final Expression[] getArgs() { return args; }
   public final int getArgCount() { return args.length; }
-  public void setFunction(Expression func) { this.func = func; }
+    public void setFunction(Expression func) { this.func = func; }
   public void setFunction(Procedure proc) { this.func = new QuoteExp(proc); }
   public void setArgs(Expression[] args) { this.args = args; }
   public Expression getArg(int i) { return args[i]; }
@@ -57,6 +78,43 @@ public class ApplyExp extends Expression
     return func instanceof QuoteExp ? ((QuoteExp) func).getValue() : null;
   }
 
+    /** Copy over splice and keyword start indexes.
+     * @param src orginal ApplyExp (may be the same as this)
+     * @aram amount to adjust indexes by
+     */
+    public void adjustSplice(ApplyExp src, int delta) {
+        if (src.firstSpliceArg >= 0)
+            this.firstSpliceArg = src.firstSpliceArg + delta;
+        if (src.firstKeywordArgIndex > 0)
+            this.firstKeywordArgIndex = src.firstKeywordArgIndex + delta;
+    }
+
+    public int spliceCount() {
+        int count = 0;
+        if (firstSpliceArg >= 0) {
+            Expression[] args = this.args;
+            int nargs = args.length;
+            for (int i = firstSpliceArg; i < nargs; i++) {
+                if (MakeSplice.argIfSplice(args[i]) != null)
+                    count++;
+            }
+        }
+        return count;
+    }
+
+    public boolean isSimple() {
+        return firstSpliceArg < 0 && firstKeywordArgIndex == 0;
+    }
+
+    public boolean isSimple(int min) {
+        return isSimple() && getArgCount() >= min;
+    }
+
+    public boolean isSimple(int min, int max) {
+        int ac = getArgCount();
+        return isSimple() && ac >= min && ac <= max;
+    }
+
     public boolean isAppendValues() {
         return func instanceof QuoteExp
             && (((QuoteExp) func).getValue()
@@ -137,74 +195,67 @@ public class ApplyExp extends Expression
     compile(exp, comp, target, false);
   }
 
-  static void compile (ApplyExp exp, Compilation comp, Target target,
-                       boolean checkInlineable)
-  {
-    int args_length = exp.args.length;
-    Expression exp_func = exp.func;
-    LambdaExp func_lambda = null;
-    String func_name = null;
-    Declaration owner = null;
-    Object quotedValue = null;
-    if (exp_func instanceof LambdaExp)
-      {
-	func_lambda = (LambdaExp) exp_func;
-	func_name = func_lambda.getName();
-	if (func_name == null)
-	  func_name = "<lambda>";
-      }
-    else if (exp_func instanceof ReferenceExp) 
-      { 
-        ReferenceExp func_ref = (ReferenceExp) exp_func;
-        owner = func_ref.contextDecl();
-        Declaration func_decl = func_ref.binding;
-        Expression dvalue;
-        while (func_decl != null && func_decl.isAlias()
-               && (dvalue = func_decl.getValueRaw()) instanceof ReferenceExp)
-          {
-            func_ref = (ReferenceExp) dvalue;
-            if (owner != null || func_decl.needsContext() || func_ref.binding == null)
-              break;
+    static void compile (ApplyExp exp, Compilation comp, Target target,
+                         boolean checkInlineable) {
+        int args_length = exp.args.length;
+        Expression exp_func = exp.func;
+        LambdaExp func_lambda = null;
+        String func_name = null;
+        Declaration owner = null;
+        Object quotedValue = null;
+        if (exp_func instanceof LambdaExp) {
+            func_lambda = (LambdaExp) exp_func;
+            func_name = func_lambda.getName();
+            if (func_name == null)
+                func_name = "<lambda>";
+        } else if (exp_func instanceof ReferenceExp) { 
+            ReferenceExp func_ref = (ReferenceExp) exp_func;
+            owner = func_ref.contextDecl();
+            Declaration func_decl = func_ref.binding;
+            Expression dval;
+            while (func_decl != null && func_decl.isAlias()
+                   && (dval = func_decl.getValueRaw()) instanceof ReferenceExp) {
+                func_ref = (ReferenceExp) dval;
+            if (owner != null || func_decl.needsContext()
+                || func_ref.binding == null)
+                break;
             func_decl = func_ref.binding;
             owner = func_ref.contextDecl();
-          }
-        if (! func_decl.getFlag(Declaration.IS_UNKNOWN))
-	  {
-	    Expression value = func_decl.getValue();
-	    func_name = func_decl.getName();
-	    if (value != null && value instanceof LambdaExp) 
-	      func_lambda = (LambdaExp) value;
-	    if (value != null && value instanceof QuoteExp) 
-              quotedValue = ((QuoteExp) value).getValue();
-	  }
-      }
-    else if (exp_func instanceof QuoteExp)
-      {
-        quotedValue = ((QuoteExp) exp_func).getValue();
-      }
-    if (checkInlineable && quotedValue instanceof Procedure)
-      {
-        Procedure proc = (Procedure) quotedValue;
-        if (target instanceof IgnoreTarget && proc.isSideEffectFree())
-          {
-            for (int i = 0; i < args_length;  i++)
-              exp.args[i].compile(comp, target);
-            return;
-          }
-        try
-          {
-            if (inlineCompile(proc, exp, comp, target))
-              return;
-          }
-        catch (Throwable ex)
-          {
-            comp.getMessages().error('e', "caught exception in inline-compiler for "+quotedValue+" - "+ex, ex);
-            return;
-          }
-      }
+            }
+            if (! func_decl.getFlag(Declaration.IS_UNKNOWN)) {
+                Expression value = func_decl.getValue();
+                func_name = func_decl.getName();
+                if (value != null && value instanceof LambdaExp) 
+                    func_lambda = (LambdaExp) value;
+                if (value != null && value instanceof QuoteExp) 
+                    quotedValue = ((QuoteExp) value).getValue();
+            }
+        } else if (exp_func instanceof QuoteExp) {
+            quotedValue = ((QuoteExp) exp_func).getValue();
+        }
+        if (checkInlineable && quotedValue instanceof Procedure) {
+            Procedure proc = (Procedure) quotedValue;
+            if (target instanceof IgnoreTarget && proc.isSideEffectFree()) {
+                for (int i = 0; i < args_length;  i++)
+                    exp.args[i].compile(comp, target);
+                return;
+            }
+            try {
+                if (inlineCompile(proc, exp, comp, target))
+                    return;
+            } catch (Error ex) {
+                throw ex;
+            } catch (Throwable ex) {
+                SourceMessages msg = comp.getMessages();
+                msg.error('f',
+                          "caught exception in inline-compiler for "
+                          +quotedValue+" - "+ex, ex);
+                throw new SyntaxException(msg);
+            }
+        }
 
-    gnu.bytecode.CodeAttr code = comp.getCode();
-    Method method;
+        CodeAttr code = comp.getCode();
+        Method method;
 
     // Check for tail-recursion.
     boolean tail_recurse
@@ -213,10 +264,13 @@ public class ApplyExp extends Expression
       // No keyword or optional arguments.
       && func_lambda.opt_args == 0 && func_lambda.keywords == null;
 
+    int spliceCount = exp.spliceCount();
+    int nonSpliceCount = args_length - spliceCount;
+        
     if (func_lambda != null)
       {
-	if ((func_lambda.max_args >= 0 && args_length > func_lambda.max_args)
-	    || args_length < func_lambda.min_args)
+	if ((func_lambda.max_args >= 0 && nonSpliceCount > func_lambda.max_args)
+	    || (nonSpliceCount < func_lambda.min_args) && spliceCount == 0)
 	  // This is supposed to get caught by InlineCalls.
 	  throw new Error ("internal error - wrong number of parameters for "
 			   + func_lambda);
@@ -231,7 +285,10 @@ public class ApplyExp extends Expression
 	    && (conv <= Compilation.CALL_WITH_CONSUMER
 		|| (conv == Compilation.CALL_WITH_TAILCALLS
 		    && ! exp.isTailCall()))
-	    && (method = func_lambda.getMethod(args_length)) != null)
+	    && (method = func_lambda.getMethod(nonSpliceCount, spliceCount)) != null
+            && (exp.firstSpliceArg < 0
+                || (PrimProcedure.takesVarArgs(method)
+                    && func_lambda.min_args <= exp.firstSpliceArg)))
 	  {
 	    PrimProcedure pproc = new PrimProcedure(method, func_lambda);
 	    boolean is_static = method.getStaticFlag();
@@ -314,7 +371,7 @@ public class ApplyExp extends Expression
       }
     */
     if (func_lambda != null && func_lambda.getInlineOnly() && !tail_recurse
-	&& func_lambda.min_args == args_length)
+	&& func_lambda.min_args == nonSpliceCount)
       {
         pushArgs(func_lambda, exp.args, exp.args.length, null, comp);
         if (func_lambda.getFlag(LambdaExp.METHODS_COMPILED))
@@ -347,7 +404,7 @@ public class ApplyExp extends Expression
 	ClassType typeContext = Compilation.typeCallContext;
 	exp_func.compile(comp, new StackTarget(Compilation.typeProcedure));
 	// evaluate args to frame-locals vars;  // may recurse!
-	if (args_length <= 4)
+	if (args_length <= 4 && exp.isSimple())
 	  {
 	    for (int i = 0; i < args_length; ++i)
 	      exp.args[i].compileWithPosition(comp, Target.pushObject);
@@ -358,7 +415,11 @@ public class ApplyExp extends Expression
 	  }
 	else
 	  {
-            compileToArray (exp.args, 0, comp);
+            if (exp.firstSpliceArg >= 0)
+                CompileArrays.createArray(Type.objectType, comp,
+                                          exp.args, 0, args_length);
+            else
+                compileToArray (exp.args, 0, comp);
 	    comp.loadCallContext();
 	    code.emitInvoke(Compilation.typeProcedure
 			    .getDeclaredMethod("checkN", 2));
@@ -381,6 +442,17 @@ public class ApplyExp extends Expression
 	return;
       }
 
+        if (exp.firstSpliceArg >= 0) {
+            Expression[] args = exp.getArgs();
+            exp_func.compile(comp, Target.pushObject);
+            CompileArrays.createArray(Type.objectType, comp,
+                                      args, 0, args.length);
+            code.emitInvoke(Compilation.typeProcedure
+                            .getDeclaredMethod("applyN", 1));
+            target.compileFromStack(comp, Type.pointer_type);
+            return;
+        }
+
     if (!tail_recurse)
       exp_func.compile (comp, new StackTarget(Compilation.typeProcedure));
 
@@ -550,16 +622,6 @@ public class ApplyExp extends Expression
       code.emitStore(vars); // rest array
   }
 
-  public final gnu.bytecode.Type getTypeRaw()
-  {
-    return type;
-  }
-
-  public final void setType (gnu.bytecode.Type type)
-  {
-    this.type = type;
-  }
-
   public boolean side_effects ()
   {
     Object value = derefFunc(func).valueIfConstant();
@@ -596,7 +658,6 @@ public class ApplyExp extends Expression
     if (afunc instanceof QuoteExp)
       {
         Object value = ((QuoteExp) afunc).getValue();
-        Inlineable compiler;
         // This logic is deprecated - instead set type during InlineCalls.
         if (value instanceof Procedure)
           type = ((Procedure) value).getReturnType(args);
@@ -608,22 +669,53 @@ public class ApplyExp extends Expression
     return type;
   }
 
-  public static Inlineable asInlineable (Procedure proc)
-  {
-    if (proc instanceof Inlineable)
-      return (Inlineable) proc;
-    return (Inlineable) Procedure.compilerKey.get(proc);
-  }
+    public static boolean isInlineable(Procedure proc) {
+        return
+            proc instanceof Inlineable
+            || Procedure.compilerKey.get(proc) != null
+            || proc.getProperty(Procedure.compilerXKey, null) != null;
+    }
 
-  static boolean inlineCompile (Procedure proc, ApplyExp exp, Compilation comp, Target target)
-    throws Throwable
-  {
-    Inlineable compiler = asInlineable(proc);
-    if (compiler == null)
-      return false;
-    compiler.compile(exp, comp, target);
-    return true;
-  }
+    static boolean inlineCompile(Procedure proc, ApplyExp exp,
+                                 Compilation comp, Target target)
+            throws Throwable {
+        if (proc instanceof PrimProcedure)
+            return ((PrimProcedure) proc).compile(exp, comp, target);
+        Object propval = proc.getProperty(Procedure.compilerXKey, null);
+        if (propval instanceof String) {
+            Object method = InlineCalls.resolveInliner(proc, (String) propval,
+                                                       compilerMethodType);
+            if (method != null)
+                propval = method;
+        }
+        /* #ifdef use:java.lang.invoke */
+        if (propval instanceof MethodHandle) {
+            return (boolean) ((MethodHandle) propval).invokeExact(exp, comp, target, proc);
+        }
+        /* #else */
+        // if (propval instanceof java.lang.reflect.Method)
+        //    return (Boolean) ((java.lang.reflect.Method) propval)
+        //        .invoke(null, new Object[] { exp, comp, target, proc });
+        /* #endif */
+        if (propval != null) {
+            comp.error('e', "compiler property string for "+proc
+                       +" is not of the form CLASS:METHOD");
+            return false;
+        }
+        if (! exp.isSimple())
+            return false;
+        Inlineable compiler;
+        if (proc instanceof Inlineable)
+            compiler = (Inlineable) proc;
+        else if ((propval = Procedure.compilerKey.get(proc)) != null)
+            compiler = (Inlineable) Procedure.compilerKey.get(proc);
+        else
+            compiler = null;
+        if (compiler == null)
+            return false;
+        compiler.compile(exp, comp, target);
+        return true;
+    }
 
   public final Expression inlineIfConstant(Procedure proc, InlineCalls visitor)
   {
@@ -661,6 +753,10 @@ public class ApplyExp extends Expression
       {
 	return new QuoteExp(proc.applyN(vals), type).setLine(this);
       }
+    catch (Error ex)
+      {
+        throw ex;
+      }
     catch (Throwable ex)
       {
 	if (messages != null)
@@ -676,4 +772,19 @@ public class ApplyExp extends Expression
       return "ApplyExp[unknownContinuation]";
     return "ApplyExp/"+(args == null?0:args.length)+'['+func+']';
   }
+
+    /* #ifdef use:java.lang.invoke */
+    static final MethodType compilerMethodType =
+        MethodType.methodType(boolean.class,
+                              gnu.expr.ApplyExp.class,
+                              gnu.expr.Compilation.class,
+                              gnu.expr.Target.class,
+                              gnu.mapping.Procedure.class);
+    /* #else */
+    // private static final Class[] compilerMethodType =
+    //     new Class[] { gnu.expr.ApplyExp.class,
+    //                   gnu.expr.Compilation.class,
+    //                   gnu.expr.Target.class,
+    //                   gnu.mapping.Procedure.class };
+    /* #endif */
 }
