@@ -3,7 +3,8 @@
 
 package gnu.expr;
 import gnu.bytecode.*;
-import gnu.mapping.*;
+import gnu.kawa.io.OutPort;
+import gnu.mapping.CallContext;
 
 /**
  * Class used to implement a block that can be exited.
