@@ -115,6 +115,8 @@ public class Compilation implements SourceLocator
     pendingImports.push(forms.lastPair());
   }
 
+    public Map<String,ModuleInfo> subModuleMap;
+
     /* Write out implicitly-compiled classes.
      * Compare javac's -implicit:class flag.  Current not set.
      */
@@ -886,6 +888,12 @@ public class Compilation implements SourceLocator
     for (int i = 0;  i < len;  i++)
       {
 	char ch = name.charAt(i);
+        if (i == 0 && ch == '$' && len >= 3 && name.charAt(1) == 'N')
+          {
+            i = 1;
+            mangled = true;
+            continue;
+          }
 	if (downCaseNext && ! reversible)
 	  {
 	    ch = Character.toLowerCase(ch);
@@ -2653,7 +2661,7 @@ public class Compilation implements SourceLocator
   }
 
   public void error(char severity, String message,
-                    String code, Declaration decl)
+                    String code, SourceLocator decl)
   {
     if (severity == 'w' && warnAsError())
       severity = 'e';
