@@ -4,6 +4,8 @@
 package gnu.expr;
 import gnu.bytecode.*;
 import gnu.kawa.reflect.OccurrenceType;
+import gnu.kawa.reflect.SingletonType;
+import gnu.kawa.lispexpr.LangPrimType;
 
 /**
  * A Target which is some variable that implements gnu.lists.Consumer.
@@ -13,19 +15,43 @@ public class ConsumerTarget extends Target
 {
   Variable consumer;
   boolean isContextTarget;
+  Type type;
 
   public ConsumerTarget(Variable consumer)
   {
     this.consumer = consumer;
+    this.type = Type.objectType;
   }
 
+  public ConsumerTarget(Variable consumer, Type type)
+  {
+    this.consumer = consumer;
+    this.type = type;
+  }
+
+    private ConsumerTarget singleTarget;
+
+    /** Get equivalent target but which only accepts a single item. */
+    public ConsumerTarget getSingleTarget() {
+        if (singleTarget == null) {
+            Type base;
+            if (! (type instanceof OccurrenceType)
+                || ! (OccurrenceType.itemCountIsOne
+                      (base = ((OccurrenceType) type).getBase())))
+                base = SingletonType.getInstance();
+            singleTarget = new ConsumerTarget(consumer, base);
+            singleTarget.isContextTarget = this.isContextTarget;
+        }
+        return singleTarget;
+    }
+
   public Variable getConsumerVariable() { return consumer; }
 
   /** True iff this target is the current CallContext's current Consumer. */
   public final boolean isContextTarget () { return isContextTarget; }
 
   /** Make a Target that uses the current CallContext's current Consumer. */
-  public static Target makeContextTarget (Compilation comp)
+  public static Target makeContextTarget (Compilation comp, Type type)
   {
     CodeAttr code = comp.getCode();
     comp.loadCallContext();
@@ -35,7 +61,7 @@ public class ConsumerTarget extends Target
     Variable result
       = scope.addVariable(code, Compilation.typeConsumer, "$result");
     code.emitStore(result);
-    ConsumerTarget target = new ConsumerTarget(result);
+    ConsumerTarget target = new ConsumerTarget(result, type);
     target.isContextTarget = true;
     return target;
   }
@@ -77,15 +103,14 @@ public class ConsumerTarget extends Target
 	code.emitInvokeStatic(makeMethod);
       }
     Variable consumer = scope.addVariable(code, ctype, null);
-    ConsumerTarget ctarget = new ConsumerTarget(consumer);
+    ConsumerTarget ctarget = new ConsumerTarget(consumer, exp.getType());
     code.emitStore(consumer);
     exp.compile(comp, ctarget);
     code.emitLoad(consumer);
     if (resultMethod != null)
       code.emitInvoke(resultMethod);
     code.popScope();
-    target.compileFromStack(comp, resultMethod == null ? ctype
-			    : resultMethod.getReturnType());
+    target.compileFromStack(comp, exp.getType());
   }
 
 
@@ -107,11 +132,23 @@ public class ConsumerTarget extends Target
     Method method = null;
     Type methodArg = null;
     boolean islong = false;
-    stackType = stackType.getImplementationType();
-    char sig;
-    if (stackType instanceof PrimType)
+    char sig; 
+    Type ttype = getType();
+    if (! stackType.isVoid()) {
+        StackTarget.convert(comp, stackType, ttype);
+        stackType = ttype;
+    }
+    // We don't want to push a character as an int (which is its
+    // implementation type) since it isn't an integer.  So we box it.
+    if (stackType == LangPrimType.characterType
+        || stackType == LangPrimType.characterOrEofType) {
+        stackType.emitCoerceToObject(code);
+        stackType = Type.objectType;
+    }
+    Type implType = stackType.getImplementationType();
+    if (implType instanceof PrimType)
       {
-	sig = stackType.getSignature().charAt(0);
+	sig = implType.getSignature().charAt(0);
 	switch (sig)
 	  {
 	  case 'B': case 'S': case 'I':
@@ -151,7 +188,7 @@ public class ConsumerTarget extends Target
     else
       {
         sig = '\0';
-	if (consumerPushed == 1 || OccurrenceType.itemCountIsOne(stackType))
+	if (consumerPushed == 1 || OccurrenceType.itemCountIsOne(implType))
           {
             methodName = "writeObject";
             methodArg = Type.pointer_type;
@@ -161,7 +198,7 @@ public class ConsumerTarget extends Target
 	    method = (Compilation.typeValues
 		      .getDeclaredMethod("writeValues", 2));
 	    code.emitLoad(consumer);
-            if (consumerPushed == 0)
+            if (consumerPushed == 0) // ??? Seems wrong - Never used.
               code.emitSwap();
 	    code.emitInvokeStatic(method);
 	    return;
@@ -172,7 +209,7 @@ public class ConsumerTarget extends Target
     else if (islong)
       {
 	code.pushScope();
-	Variable temp = code.addLocal(stackType);
+	Variable temp = code.addLocal(implType);
 	code.emitStore(temp);
 	code.emitLoad(consumer);
 	code.emitLoad(temp);
@@ -199,18 +236,20 @@ public class ConsumerTarget extends Target
   {
     Type stackType = exp.getType();
     Type implType = stackType.getImplementationType();
-    if ((implType instanceof PrimType && ! implType.isVoid())
+    if ((implType instanceof PrimType && ! implType.isVoid()
+         && stackType != LangPrimType.characterType
+         && stackType != LangPrimType.characterOrEofType)
         || gnu.kawa.reflect.OccurrenceType.itemCountIsOne(implType))
       {
         // Optimization to avoid a 'swap'.
         comp.getCode().emitLoad(this.consumer);
-        Target starget = StackTarget.getInstance(stackType);
-        exp.compile(comp, starget);
-        compileFromStack(comp, implType, 1);
+        Type ttype = this.type;
+        exp.compile(comp, StackTarget.getInstance(ttype));
+        compileFromStack(comp, ttype, 1);
         return true;
       }
     return false;
    }
 
-  public Type getType() { return Compilation.scmSequenceType; }
+  public Type getType() { return type; }
 }
