@@ -3,6 +3,8 @@
 
 package gnu.expr;
 import gnu.bytecode.*;
+import gnu.kawa.functions.*;
+import gnu.kawa.io.OutPort;
 import gnu.mapping.*;
 import gnu.text.SourceLocator;
 import gnu.math.*;
@@ -10,7 +12,6 @@ import java.util.ArrayList;
 import java.util.List;
 import java.lang.reflect.Proxy;
 import java.lang.annotation.ElementType;
-import gnu.kawa.functions.*;
 
 /**
  * The static information associated with a local variable binding.
@@ -417,7 +418,7 @@ public class Declaration
                 int column = access.getColumnNumber();
                 Label startTry = new Label(code);
                 startTry.define(code);
-                code.emitInvokeVirtual(Compilation.getLocationMethod);
+                code.emitInvokeVirtual(getLocationMethod);
                 Label endTry = new Label(code);
                 endTry.define(code);
                 Label endLabel = new Label(code);
@@ -444,7 +445,7 @@ public class Declaration
                   code.endFragment(fragment_cookie);
               }
             else
-              code.emitInvokeVirtual(Compilation.getLocationMethod);
+              code.emitInvokeVirtual(getLocationMethod);
 
             rtype = Type.pointer_type;
           }
@@ -452,6 +453,10 @@ public class Declaration
     target.compileFromStack(comp, rtype);
   }
 
+    static final Method getLocationMethod
+        = Compilation.typeLocation.addMethod("get", Type.typeArray0,
+                                             Type.objectType, Access.PUBLIC);
+
   /* Compile code to store a value (which must already be on the
      stack) into this variable. */
   public void compileStore (Compilation comp)
@@ -528,7 +533,7 @@ public class Declaration
   // This should be a type property, not a variable property, at some point!
   public static final int IS_SINGLE_VALUE = 0x40000;
 
-  /** This flag bit is set if this can be be acceessed from other modules.
+  /** This flag bit is set if this can be be accessed from other modules.
    * Ignored unless PRIVATE.
    * Used when an exported macro references a non-exported name. */
   public static final int EXTERNAL_ACCESS = 0x80000;
@@ -687,11 +692,10 @@ public class Declaration
     setFlag(indirectBinding, INDIRECT_BINDING);
   }
 
-  public void maybeIndirectBinding (Compilation comp)
-  {
-      if (isLexical() && ! inExternalModule(comp))
-      setIndirectBinding(true);
-  }
+    public void maybeIndirectBinding(Compilation comp) {
+        if (isLexical() && ! inExternalModule(comp) && !getFlag(TYPE_SPECIFIED))
+            setIndirectBinding(true);
+    }
 
     public boolean inExternalModule(Compilation comp) {
         return context instanceof ModuleExp && context != comp.mainLambda;
@@ -856,6 +860,32 @@ public class Declaration
     return annotations.get(i);
   }
 
+    public <T extends java.lang.annotation.Annotation>
+    T getAnnotation(Class<T> clas) {
+        int n = numAnnotations();
+        for (int i = 0;  i < n;  i++) {
+            Object ann = getAnnotation(i).valueIfConstant();
+            if (clas.isInstance(ann))
+                return (T) ann;
+        }
+        return null;
+    }
+
+    public AnnotationEntry getAnnotation(String className) {
+        int n = numAnnotations();
+        for (int i = 0;  i < n;  i++) {
+            Object ann = getAnnotation(i).valueIfConstant();
+            if (ann instanceof AnnotationEntry) {
+                AnnotationEntry ae =
+                    (AnnotationEntry) Proxy.getInvocationHandler(ann);
+                if (className.equals(ae.getAnnotationType())) {
+                    return ae;
+                }
+            }
+        }
+        return null;
+    }
+
   /** Replace one of the annotations associated with this declaration. */
   public void setAnnotation (int i, Expression ann)
   {
@@ -1170,6 +1200,14 @@ public class Declaration
         while (frameType.getDeclaredField(fname) != null)
           fname = fname.substring(0, nlength) + '$' + (++ counter);
         field = frameType.addField (fname, ftype, fflags);
+        if (getAnnotation(kawa.SourceType.class) == null) {
+            String encType = comp.getLanguage().encodeType(getType());
+            if (encType != null && encType.length() > 0) {
+                AnnotationEntry ae = new AnnotationEntry(ClassType.make("kawa.SourceType"));
+                ae.addMember("value", encType, Type.javalangStringType);
+                RuntimeAnnotationsAttr.maybeAddAnnotation(field, ae);
+            }
+        }
         if (haveName)
           {
             Object fsymbol = getSymbol();
@@ -1309,7 +1347,7 @@ public class Declaration
                 String fname = Compilation.mangleNameIfNeeded(name);
                 procField = procType.getDeclaredField(fname);
               }
-            catch (Throwable ex)
+            catch (Exception ex)
               {
                 System.err.println("CAUGHT "+ex+" in getDeclaration for "+proc);
                 return null;
@@ -1385,7 +1423,7 @@ public class Declaration
                 noteValue(value);
                 return value;
               }
-            catch (Throwable ex)
+            catch (Exception ex)
               {
               }
           }
@@ -1505,6 +1543,17 @@ public class Declaration
       noteValue(new ValueSource(ValueSource.APPLY_KIND, app, index));
   }
 
+    /** Set symbol from initializing SetExp.
+     * Used for an export-only alias, for handling export-with-rename.
+     */
+    public boolean patchSymbolFromSet() {
+        if (nvalues != 1 || values[0].kind != ValueSource.SET_RHS_KIND)
+            return false;
+        SetExp sexp = (SetExp) values[0].base;
+        setSymbol(((SetExp) values[0].base).getSymbol());
+        return true;
+    }
+
   private void checkNameDecl (Expression value)
   {
     if (nvalues == 1 && values[0].kind == ValueSource.GENERAL_KIND)
