@@ -5,6 +5,8 @@ import gnu.text.Printable;
 import gnu.text.SourceLocator;
 import gnu.lists.Consumer;
 import java.io.PrintWriter;
+import gnu.kawa.io.CharArrayOutPort;
+import gnu.kawa.io.OutPort;
 import gnu.kawa.util.IdentityHashTable;
 import gnu.kawa.reflect.OccurrenceType;
 
@@ -274,7 +276,7 @@ public abstract class Expression extends Procedure0
     parser.loopEnter();
     parser.loopCond(parser.parse(cond));
     parser.loopBody(parser.parse(body));
-    return parser.loopRepeat();
+    return parser.loopRepeatDone();
   }
   
   /** Copies the current location. */
@@ -290,6 +292,12 @@ public abstract class Expression extends Procedure0
     return this;
   }
 
+    public final Expression maybeSetLine(Expression old) {
+        if (position == 0 && old != null && old.position != 0)
+            setLocation(old);
+        return this;
+    }
+
   public final void setFile (String filename)
   {
     this.filename = filename;
@@ -369,6 +377,14 @@ public abstract class Expression extends Procedure0
     return Type.pointer_type;
   }
 
+    public final Type getTypeRaw() {
+        return type;
+    }
+
+    public void setType(Type type) {
+        this.type = type;
+    }
+
     /** True if the expression provably never returns.
      * Currently, this is very limited, not handling infinite recursion
      * (tail- or otherwise), but better analysis is planned.
@@ -377,6 +393,19 @@ public abstract class Expression extends Procedure0
         return getType() == Type.neverReturnsType;
     }
 
+    /** Is this a keyword argument?
+     * In the future this will return non-null only for explicit
+     * literal non-quoted Keywords.
+     */
+    public Keyword checkLiteralKeyword() {
+        if (this instanceof QuoteExp) {
+            Object val = ((QuoteExp) this).getValue();
+            if (val instanceof Keyword)
+                return (Keyword) val;
+        }
+        return null;
+    }
+
   public boolean isSingleValue()
   {
     return OccurrenceType.itemCountIsOne(getType());
