@@ -5,12 +5,16 @@ package gnu.expr;
 import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.kawa.lispexpr.LangObjType;
+import gnu.kawa.reflect.CompileArrays;
 import gnu.lists.ConsumerWriter;
+import kawa.SourceMethodType;
 import java.io.Writer;
+import java.lang.reflect.Array;
+import gnu.kawa.functions.MakeSplice;
 
 /** A primitive Procedure implemented by a plain Java method. */
 
-public class PrimProcedure extends MethodProc implements Inlineable {
+public class PrimProcedure extends MethodProc {
     private Type retType;
 
     /** The types of the method parameters.
@@ -47,6 +51,11 @@ public class PrimProcedure extends MethodProc implements Inlineable {
 
     public Type getReturnType (Expression[] args) { return retType; }
 
+    public ClassType getDeclaringClass() {
+        return methodForInvoke == null ? null
+            : methodForInvoke.getDeclaringClass();
+    }
+
     public Method getMethod () { return method; }
 
     public void setMethodForInvoke(Method m) {
@@ -64,18 +73,20 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     sideEffectFree = true;
   }
 
-  /** Return true iff the last parameter is a "rest" argument. */
-  public boolean takesVarArgs()
-  {
-    if (method != null)
-      {
-        if ((method.getModifiers() & Access.VARARGS) != 0)
-          return true;
-	String name = method.getName();
-	return name.endsWith("$V") || name.endsWith("$V$X");
-      }
-    return false;
-  }
+    /** Return true iff the last parameter is a "rest" argument. */
+    public boolean takesVarArgs() {
+        return takesVarArgs(method);
+    }
+
+    public static boolean takesVarArgs(Method method) {
+        if (method != null) {
+            if ((method.getModifiers() & Access.VARARGS) != 0)
+                return true;
+            String name = method.getName();
+            return name.endsWith("$V") || name.endsWith("$V$X");
+        }
+        return false;
+    }
 
   public boolean takesContext()
   {
@@ -87,11 +98,25 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     return method.getName().endsWith("$X");
   }
 
-  public int isApplicable(Type[] argTypes)
-  {
-    int app = super.isApplicable(argTypes);
+    /** Support passing an explicit array to a varargs function.
+     * This is a kludge inherited from Java to support backwards
+     * compatibility after various methods were converte to take varargs.
+     * If Java5-style VARARS we allow both a variable-length argument list,
+     * or if the last argument already is an array we can use it as is.
+     * The tricky part is we sometimes have to distinguish these cases
+     * at run-time - see the logic for createVarargsArrayIfNeeded in
+     * compileArgs.
+     * FIXME This is needless and unreliable complexity.  We should by default
+     * create a varargs array - even if the actual argument is an array.
+     * People should now use splices instead.
+     */
+    public static boolean explicitArrayAsVarArgsAllowed = true;
+
+    public int isApplicable(Type[] argTypes, Type restType) {
+        int app = super.isApplicable(argTypes, restType);
     int nargs = argTypes.length;
-    if (app == -1 && method != null
+    if (explicitArrayAsVarArgsAllowed
+        && app == -1 && method != null && restType == null
         && (method.getModifiers() & Access.VARARGS) != 0
         && nargs > 0 && argTypes[nargs-1] instanceof ArrayType)
       {
@@ -100,7 +125,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
         Type[] tmp = new Type[nargs];
         System.arraycopy(argTypes, 0, tmp, 0, nargs-1);
         tmp[nargs-1] = ((ArrayType) argTypes[nargs-1]).getComponentType();
-        return super.isApplicable(tmp);
+        return super.isApplicable(tmp, null);
       }
     return app;
   }
@@ -174,100 +199,89 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     return matchN(args, ctx);
   }
 
-  public int matchN (Object[] args, CallContext ctx)
-  {
-    int nargs = args.length;
-    boolean takesVarArgs = takesVarArgs();
-    int fixArgs = minArgs();
-    if (nargs < fixArgs)
-      return NO_MATCH_TOO_FEW_ARGS|fixArgs;
-    if (! takesVarArgs && nargs > fixArgs)
-      return NO_MATCH_TOO_MANY_ARGS|fixArgs;
-    int paramCount = argTypes.length;
-    Type elementType = null;
-    Object[] restArray = null;
-    int extraCount = (takesTarget() || isConstructor()) ? 1 : 0;
-    boolean takesContext = takesContext();
-    Object[] rargs = new Object[paramCount];
-    if (takesContext)
-      rargs[--paramCount] = ctx;
-    Object extraArg;
-    if (takesVarArgs)
-      {
-	Type restType = argTypes[paramCount-1];
-        if (restType == Compilation.scmListType || restType == LangObjType.listType)
-	  { // FIXME
-	    rargs[paramCount-1] = gnu.lists.LList.makeList(args, fixArgs);
-	    nargs = fixArgs;
-            elementType = Type.objectType;
-	  }
-	else
-	  {
-	    ArrayType restArrayType = (ArrayType) restType;
-	    elementType = restArrayType.getComponentType();
-	    Class elementClass = elementType.getReflectClass();
-	    restArray = (Object[])
-	      java.lang.reflect.Array.newInstance(elementClass, nargs-fixArgs);
-	    rargs[paramCount-1] = restArray;
-	  }
-      }
-    if (isConstructor())
-      extraArg = args[0];
-    else if (extraCount != 0)
-      {
-	try
-	  {
-            extraArg = method.getDeclaringClass().coerceFromObject(args[0]);
-	  }
-	catch (ClassCastException ex)
-          {
-            return NO_MATCH_BAD_TYPE|1;
-          }
-      }
-    else
-      extraArg = null;
-    for (int i = extraCount;  i < args.length; i++)
-      {
-        Object arg = args[i];
-        Type type = i < fixArgs ? argTypes[i-extraCount] : elementType;
-        if (type != Type.objectType)
-          {
-            try
-              {
-                arg = type.coerceFromObject(arg);
-              }
-            catch (ClassCastException ex)
-              {
-                return NO_MATCH_BAD_TYPE|(i+1);
-              }
-          }
-        if (i < fixArgs)
-          rargs[i-extraCount] = arg;
-        else if (restArray != null) // I.e. using array rather than LList.
-          restArray[i - fixArgs] = arg;
-      }
-    ctx.value1 = extraArg;
-    ctx.values = rargs;
-    ctx.proc = this;
-    return 0;
-  }
+    public int matchN(Object[] args, CallContext ctx) {
+        int nargs = args.length;
+        boolean takesVarArgs = takesVarArgs();
+        int fixArgs = minArgs();
+        if (nargs < fixArgs)
+            return NO_MATCH_TOO_FEW_ARGS|fixArgs;
+        if (! takesVarArgs && nargs > fixArgs)
+            return NO_MATCH_TOO_MANY_ARGS|fixArgs;
+        int paramCount = argTypes.length;
+        Type elementType = null;
+        Object restArray = null;
+        int extraCount = (takesTarget() || isConstructor()) ? 1 : 0;
+        boolean takesContext = takesContext();
+        Object[] rargs = new Object[paramCount];
+        if (takesContext)
+            rargs[--paramCount] = ctx;
+        Object extraArg;
+        if (takesVarArgs) {
+            Type restType = argTypes[paramCount-1];
+            if (restType == Compilation.scmListType || restType == LangObjType.listType) {
+                // FIXME
+                rargs[paramCount-1] = gnu.lists.LList.makeList(args, fixArgs);
+                nargs = fixArgs;
+                elementType = Type.objectType;
+            } else {
+                ArrayType restArrayType = (ArrayType) restType;
+                elementType = restArrayType.getComponentType();
+                Class elementClass = elementType.getReflectClass();
+                restArray = Array.newInstance(elementClass, nargs-fixArgs);
+                rargs[paramCount-1] = restArray;
+            }
+        }
+        if (isConstructor())
+            extraArg = args[0];
+        else if (extraCount != 0) {
+            try {
+                extraArg = getDeclaringClass().coerceFromObject(args[0]);
+            } catch (ClassCastException ex) {
+                return NO_MATCH_BAD_TYPE|1;
+            }
+        } else
+            extraArg = null;
+        for (int i = extraCount;  i < args.length; i++) {
+            Object arg = args[i];
+            Type type = i < fixArgs ? argTypes[i-extraCount]
+                : elementType == null ? null : elementType;
+            if (type != Type.objectType) {
+                try {
+                    arg = type.coerceFromObject(arg);
+                } catch (ClassCastException ex) {
+                    return NO_MATCH_BAD_TYPE|(i+1);
+                }
+            }
+            if (i < fixArgs)
+                rargs[i-extraCount] = arg;
+            else if (restArray != null) { // I.e. using array rather than LList.
+                if (type instanceof PrimType)
+                    arg = ((PrimType) type).convertToRaw(arg);
+                Array.set(restArray, i - fixArgs, arg);
+            }
+        }
+        ctx.value1 = extraArg;
+        ctx.values = rargs;
+        ctx.proc = this;
+        return 0;
+    }
 
   public void apply (CallContext ctx) throws Throwable
   {
     int arg_count = argTypes.length;
     boolean is_constructor = isConstructor();
-    boolean slink = is_constructor && method.getDeclaringClass().hasOuterLink();
+    boolean slink = is_constructor && getDeclaringClass().hasOuterLink();
 
     try
       {
 	if (member == null)
 	  {
-	    Class clas = method.getDeclaringClass().getReflectClass();
+	    Class clas = getDeclaringClass().getReflectClass();
 	    Class[] paramTypes = new Class[arg_count+(slink?1:0)];
 	    for (int i = arg_count; --i >= 0; )
 	      paramTypes[i+(slink?1:0)] = argTypes[i].getReflectClass();
             if (slink)
-              paramTypes[0] = method.getDeclaringClass().getOuterLinkType().getReflectClass();
+              paramTypes[0] = getDeclaringClass().getOuterLinkType().getReflectClass();
 	    if (is_constructor)
 	      member = clas.getConstructor(paramTypes);
 	    else if (method != Type.clone_method)
@@ -333,92 +347,60 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     this(method, '\0', language, null);
   }
 
-    static Type resolveTypeVariables(Type langType, ParameterizedType parameterizedType) {
-        if (langType instanceof TypeVariable)
-            return resolveTypeVariable((TypeVariable) langType, parameterizedType);
-        if (langType instanceof ParameterizedType) {
-            ParameterizedType ptype = (ParameterizedType) langType;
-            Type[] paramTypes = ptype.getTypeArgumentTypes();
-            int nparams = paramTypes.length;
-            Type[] resolvedTypes = new Type[nparams];
-            boolean changed = false;
-            for (int i = 0; i < nparams;  i++) {
-                Type t0 = paramTypes[i];
-                char bound = ptype.getTypeArgumentBound(i);
-                // FIXME Don't support wildcards here yet.
-                if (bound != '\0')
-                    return langType.getRawType();
-                Type t1 = resolveTypeVariables(t0, parameterizedType);
-                resolvedTypes[i] = t1;
-                if (t0 != t1)
-                    changed = true;
-            }
-            if (changed) {
-                return new ParameterizedType(ptype.getRawType(), resolvedTypes);
+    public PrimProcedure(Method method, char mode, Language language,
+			 ParameterizedType parameterizedType) {
+        this.mode = mode;
+
+        init(method);
+        // This stuff deals with that a language may have its own mapping
+        // from Java types to language types, for coercions and other reasons.
+        Type[] pTypes = this.argTypes;
+        int nTypes = pTypes.length;
+        argTypes = null;
+        String[] annotTypes;
+        try {
+            SourceMethodType sourceType = method.getAnnotation(SourceMethodType.class);
+            annotTypes = sourceType == null ? null : sourceType.value();
+        } catch (Throwable ex) {
+            annotTypes = null;
+        }
+        for (int i = nTypes;  --i >= 0; ) {
+            Type javaType = pTypes[i];
+            Type langType = decodeType(javaType, annotTypes, i+1,
+                                       parameterizedType, language);
+            if (javaType != langType) {
+                if (argTypes == null) {
+                    argTypes = new Type[nTypes];
+                    System.arraycopy(pTypes, 0, argTypes, 0, nTypes); 
+                }
+                argTypes[i] = langType;
             }
         }
-        return langType;
-}
-
-    static Type resolveTypeVariable(TypeVariable tvar, ParameterizedType parameterizedType) {
-	if (parameterizedType != null) {
-	    TypeVariable[] tparams = parameterizedType.getRawType().getTypeParameters();
-	    int nparams = tparams.length;
-	    for (int i = 0;  i < nparams;  i++) {
-		if (tvar.getName().equals(tparams[i].getName())) {
-		    return parameterizedType.getTypeArgumentType(i);
-		}
-	    }
-	}
-	return tvar.getRawType();
+        if (argTypes == null)
+            argTypes = pTypes;
+        if (isConstructor())
+            retType = getDeclaringClass();
+        else if (method.getName().endsWith("$X"))
+            retType = Type.objectType;
+        else {
+            retType = decodeType(method.getReturnType(),
+                                 annotTypes, 0, parameterizedType, language);
+            // Kludge - toStringType doesn't have methods.
+            // It shouldn't be used as the "type" of anything -
+            // it's just a type with a coercion.  FIXME.
+            if (retType == Type.toStringType)
+                retType = Type.javalangStringType;
+        }
     }
 
-    public PrimProcedure(Method method, char mode, Language language,
-			 ParameterizedType parameterizedType)
-  {
-    this.mode = mode;
+    static Type decodeType(Type javaType, String[] annotTypes, int annotIndex,
+                           ParameterizedType parameterizedType,
+                           Language lang) {
+        String annotType = annotTypes != null && annotTypes.length > annotIndex
+            ? annotTypes[annotIndex] : null;
+        return lang.decodeType(javaType, annotType, parameterizedType);
+    }
 
-    init(method);
-    // This stuff deals with that a language may have its own mapping
-    // from Java types to language types, for coercions and other reasons.
-    Type[] pTypes = this.argTypes;
-    int nTypes = pTypes.length;
-    argTypes = null;
-    for (int i = nTypes;  --i >= 0; )
-      {
-	Type javaType = pTypes[i];
-	Type langType = resolveTypeVariables(javaType, parameterizedType);
-	langType = language.getLangTypeFor(langType);
-	if (javaType != langType)
-	  {
-	    if (argTypes == null)
-	      {
-		argTypes = new Type[nTypes];
-		System.arraycopy(pTypes, 0, argTypes, 0, nTypes); 
-	      }
-	    argTypes[i] = langType;
-	  }
-      }
-    if (argTypes == null)
-      argTypes = pTypes;
-    if (isConstructor())
-      retType = method.getDeclaringClass();
-    else if (method.getName().endsWith("$X"))
-      retType = Type.objectType;
-    else
-      {
-        retType = method.getReturnType();
-        retType = resolveTypeVariables(retType, parameterizedType);
-        retType = language.getLangTypeFor(retType);
-
-        // Kludge - toStringType doesn't have methods.
-        // It shouldn't be used as the "type" of anything -
-        // it's just a type with a coercion.  FIXME.
-        if (retType == Type.toStringType)
-          retType = Type.javalangStringType;
-      }
-  }
-  
     private void setOpcode(Method m) {
         int flags = m.getModifiers();
         if ((flags & Access.STATIC) != 0)
@@ -518,7 +500,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
    *   pass a link to a closure environment, which was pushed by our caller.)
    *   If thisType==null, no special handling of args[0] or argTypes[0].
    */
-  private void compileArgs(Expression[] args, int startArg, Type thisType, Compilation comp)
+    private void compileArgs(ApplyExp exp, Expression[] args, int startArg, Type thisType, Compilation comp)
  {
     boolean variable = takesVarArgs();
     String name = getName();
@@ -531,14 +513,13 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     int nargs = args.length - startArg;
     boolean is_static = thisType == null || skipArg != 0;
 
-    // If Java5-style VARARS we allow both a variable-length argument list,
-    // or if the last argument already is an array we can use it as is.
-    // The tricky part is we sometimes have to distinguish these cases
-    // at run-time, in which case we set createVarargsArrayIfNeeded.
-    // FIXME This is needless and unreliable complexity.  We should by default
-    // create a varargs array - even if the actual argument is an array.
+    // Do we need to check at runtime whether a final argument to a VARARGS
+    // is an array or need to be wrapped in an array?
+    // See comment at explicitArrayAsVarArgsAllowed.
     boolean createVarargsArrayIfNeeded = false;
-    if (variable && (method.getModifiers() & Access.VARARGS) != 0
+    if (explicitArrayAsVarArgsAllowed
+        && variable && (method.getModifiers() & Access.VARARGS) != 0
+        && exp.firstSpliceArg < 0
         && nargs > 0 && argTypes.length > 0
         && nargs == arg_count + (is_static ? 0 : 1))
       {
@@ -566,9 +547,32 @@ public class PrimProcedure extends MethodProc implements Inlineable {
 		gnu.kawa.functions.MakeList.compile(args, startArg+i, comp);
 		break;
 	      }
-            code.emitPushInt(args.length - startArg - fix_arg_count);
+            if (startArg+i+1== args.length
+                && exp.firstSpliceArg==startArg+i) {
+                // See if final argument is a splice of an array we can re-use.
+                Expression spliceArg = MakeSplice.argIfSplice(args[startArg+i]);
+                Type spliceType = spliceArg.getType();
+                if (spliceType instanceof ArrayType) {
+                    Type spliceElType = ((ArrayType) spliceType).getComponentType();
+                    Type argElType = ((ArrayType) arg_type).getComponentType();
+                    if (spliceElType == argElType
+                        || (argElType == Type.objectType
+                            && spliceElType instanceof ObjectType)
+                        || (argElType instanceof ClassType
+                            && spliceElType instanceof ClassType
+                            && spliceElType.isSubtype(argElType))) {
+                        spliceArg.compileWithPosition(comp, Target.pushObject);
+                        i = nargs;
+                        break;
+                    }
+                }
+            }
+                
             arg_type = ((ArrayType) arg_type).getComponentType();
-            code.emitNewArray(arg_type);
+            CompileArrays.createArray(arg_type, comp,
+                                      args, startArg+i, args.length);
+            i = nargs;
+            break;
           }
         if (i >= nargs)
           break;
@@ -589,7 +593,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
 	  source == null ? CheckedTarget.getInstance(argTypeForTarget, name, i+1)
 	  : CheckedTarget.getInstance(argTypeForTarget, source, i);
 	args[startArg+i].compileNotePosition(comp, target, args[startArg+i]);
-        if (createVarargsNow)
+        if (createVarargsNow) // Only if explicitArrayAsVarArgsAllowed
           {
             // Wrap final argument in array if not already an array:
             // Emit: (arg instanceof T[] ? (T[]) arg : new T[]{arg})
@@ -606,7 +610,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
             code.emitPushInt(0); // Stack array array value 0
             code.emitSwap(); // Stack: array array 0 value
             eltype.emitCoerceFromObject(code);
-            code.emitArrayStore(arg_type); // Stack: array
+            code.emitArrayStore(eltype); // Stack: array
             code.emitFi();
           }
         if (i >= fix_arg_count)
@@ -616,10 +620,17 @@ public class PrimProcedure extends MethodProc implements Inlineable {
       }
   }
 
-  public void compile (ApplyExp exp, Compilation comp, Target target)
-  {
+    public boolean compile(ApplyExp exp, Compilation comp, Target target) {
+        if (exp.firstKeywordArgIndex != 0)
+            return false;
+        // We can optimize splice arguments if they're in the "varargs"
+        // section of the argument list.
+        if (exp.firstSpliceArg >= 0
+            && (! takesVarArgs() || minArgs() > exp.firstSpliceArg))
+            return false;
+
     gnu.bytecode.CodeAttr code = comp.getCode();
-    ClassType mclass = method == null ? null : methodForInvoke.getDeclaringClass();
+    ClassType mclass = getDeclaringClass();
     Expression[] args = exp.getArgs();
     if (isConstructor())
       {
@@ -660,29 +671,30 @@ public class PrimProcedure extends MethodProc implements Inlineable {
             comp.letEnter();
             LetExp let = comp.letDone(new ApplyExp(exp.func, xargs));
             let.compile(comp, target);
-            return;
+            return true;
           }
         code.emitNew(mclass);
         code.emitDup(mclass);
       }
-    String arg_error = WrongArguments.checkArgCount(this, args.length);
+    int spliceCount = exp.spliceCount();
+    String arg_error = WrongArguments.checkArgCount(this,
+                                                    args.length-spliceCount,
+                                                    spliceCount>0);
     if (arg_error != null)
       comp.error('e', arg_error);
 
     compile(getStaticFlag() ? null : mclass, exp, comp, target);
+    return true;
   }
 
   void compile (Type thisType, ApplyExp exp, Compilation comp, Target target)
   {
     Expression[] args = exp.getArgs();
     gnu.bytecode.CodeAttr code = comp.getCode();
-    Type stackType = retType;
-    if (method != null && method.getReturnType() instanceof TypeVariable)
-	stackType = method.getReturnType().getRawType();
     int startArg = 0;
     if (isConstructor())
       {
-        ClassType mclass = method == null ? null :  method.getDeclaringClass();
+        ClassType mclass = getDeclaringClass();
         if (mclass.hasOuterLink())
           {
             ClassExp.loadSuperStaticLink(args[0], mclass, comp);
@@ -695,7 +707,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     else if (opcode() == 183 && mode == 'P' && "<init>".equals(method.getName()))
       {
         // Specifically handle passing a static-link.
-        ClassType mclass = method == null ? null :  method.getDeclaringClass();
+        ClassType mclass = getDeclaringClass();
         if (mclass.hasOuterLink())
           {
             code.emitPushThis();
@@ -707,17 +719,17 @@ public class PrimProcedure extends MethodProc implements Inlineable {
       }
     else if (takesTarget() && method.getStaticFlag())
       startArg = 1;
-    compileArgs(args, startArg, thisType, comp);
+    compileArgs(exp, args, startArg, thisType, comp);
 
     if (method == null)
       {
         code.emitPrimop (opcode(), args.length, retType);
-        target.compileFromStack(comp, stackType);
+        target.compileFromStack(comp, retType);
       }
     else
       {
         compileInvoke(comp, methodForInvoke, target,
-                      exp.isTailCall(), op_code, stackType);
+                      exp.isTailCall(), op_code, retType);
       }
   }
 
@@ -727,7 +739,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
    */
   public static void
   compileInvoke (Compilation comp, Method method, Target target,
-                 boolean isTailCall, int op_code, Type stackType)
+                 boolean isTailCall, int op_code, Type returnType)
   {
     CodeAttr code = comp.getCode();
     comp.usedClass(method.getDeclaringClass());
@@ -780,7 +792,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     else
       {
         comp.loadCallContext();
-        stackType = Type.objectType;
+        returnType = Type.objectType;
         code.pushScope();
         Variable saveIndex = code.addLocal(Type.intType);
         comp.loadCallContext();
@@ -808,8 +820,23 @@ public class PrimProcedure extends MethodProc implements Inlineable {
         // that take a context (and whose return type is thus void).
         compileReachedUnexpected(code);
       }
-    else
-      target.compileFromStack(comp, stackType);
+    else {
+        if (method.getReturnType() instanceof TypeVariable) {
+            if (returnType instanceof ClassType) {
+                // This avoids an unneeded exception handler (if the
+                // target is a CheckedTarget), and a needless call to
+                // Promise.force.  Also, if the target.getType() is
+                // different from the returnType, it's probably better
+                // to convert to returnType first - it might make it
+                // easier to find the right conversion, or emit a more
+                // accurate error message.
+                code.emitCheckcast(returnType);
+            } else
+                returnType = method.getReturnType().getRawType();
+        }
+
+      target.compileFromStack(comp, returnType);
+    }
   }
 
     public static void compileReachedUnexpected(CodeAttr code) {
@@ -826,7 +853,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
       {
         if (index == 0)
           return isConstructor() ? Type.objectType
-            : method.getDeclaringClass();
+            : getDeclaringClass();
         index--;
       }
     int lenTypes = argTypes.length;
@@ -939,7 +966,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
 	pproc = null;
 	for (int i = gproc.count;  --i >= 0; )
 	  {
-	    int applic = methods[i].isApplicable(atypes);
+              int applic = methods[i].isApplicable(atypes, null/*FIXME*/);
 	    if (applic < 0)
 	      continue;
 	    if (pproc != null)
@@ -952,7 +979,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
     if (pproc instanceof PrimProcedure)
       {
 	PrimProcedure prproc = (PrimProcedure) pproc;
-	if (prproc.isApplicable(atypes) >= 0)
+	if (prproc.isApplicable(atypes, null/*FIXME*/) >= 0)
 	  return prproc;
       }
     Class pclass = getProcedureClass(pproc);
@@ -1002,7 +1029,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
       cl = ((ModuleMethod) proc).module.getClass();
     else if (proc instanceof PrimProcedure)
       {
-        Method pmethod = ((PrimProcedure) proc).method;
+        Method pmethod = ((PrimProcedure) proc).methodForInvoke;
         if (pmethod != null)
           {
             cl = pmethod.getDeclaringClass().getReflectClass();
@@ -1010,6 +1037,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
           }
       }
     ClassLoader loader = cl.getClassLoader();
+    if (loader == null) loader = ClassLoader.getSystemClassLoader();
     String cname = cl.getName();
     String rname = cname.replace('.', '/') + ".class";
     ClassType ctype = new ClassType();
@@ -1140,7 +1168,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
 	      }
 	    PrimProcedure prproc = new PrimProcedure(meth, language);
 	    prproc.setName(name);
-	    int code = prproc.isApplicable(atypes);
+	    int code = prproc.isApplicable(atypes, null/*FIXME*/);
 	    if (code < 0 || code < bestCode)
 	      continue;
 	    if (code > bestCode)
@@ -1187,7 +1215,7 @@ public class PrimProcedure extends MethodProc implements Inlineable {
       }
     else
       {
-	buf.append(method.getDeclaringClass().getName());
+	buf.append(getDeclaringClass().getName());
 	buf.append('.');
 	buf.append(method.getName());
       }
