@@ -3,10 +3,12 @@
 
 package gnu.jemacs.buffer;
 import java.io.*;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.Path;
 import gnu.mapping.*;
 import gnu.lists.*;
-import gnu.text.*;
 import gnu.commonlisp.lang.Symbols; // FIXME
+import gnu.text.Char;
 
 public abstract class Buffer extends AbstractSequence implements CharSeq
 {
@@ -87,7 +89,11 @@ public abstract class Buffer extends AbstractSequence implements CharSeq
     getStringContent().setCharAt(index, ch);
   }
 
-  /**
+    public void setCharacterAt(int index, int ch) {
+        getStringContent().setCharacterAt(index, ch);
+    }
+
+ /**
    * @see gnu.lists.CharSeq#fill(char)
    */
   public void fill(char value)
