@@ -2,6 +2,8 @@ package gnu.jemacs.swing;
 import gnu.jemacs.buffer.*;
 import gnu.mapping.*;
 import java.io.*;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.NullReader;
 import gnu.lists.CharBuffer;
 
 public class MarkerReader extends InPort
@@ -21,7 +23,7 @@ public class MarkerReader extends InPort
 
   MarkerReader(Buffer buffer, Marker marker)
   {
-    super(gnu.text.NullReader.nullReader, buffer.getPath());
+    super(NullReader.nullReader, buffer.getPath());
     this.marker = marker;
     content = ((gnu.jemacs.swing.SwingBuffer) buffer).content.buffer;
     this.buffer = content.getArray();
