@@ -4,6 +4,7 @@ import kawa.standard.Scheme;
 import gnu.lists.*;
 import gnu.xml.*;
 import gnu.expr.*;
+import gnu.kawa.io.InPort;
 import gnu.kawa.lispexpr.ReadTable;
 
 public class BRL extends Scheme
@@ -51,7 +52,7 @@ public class BRL extends Scheme
 	loadClass("gnu.kawa.brl.progfun");
 	loadClass("gnu.kawa.servlet.HTTP");
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
 	System.err.println("caught "+ex);
       }
