@@ -62,13 +62,16 @@ public class CompileNamedPart
           }
         if (CompileReflect.checkKnownClass(typeval, comp) < 0)
           return exp;
+        ObjectType otype = (ObjectType) typeval;
         PrimProcedure[] methods
-          = ClassMethods.getMethods((ObjectType) typeval,
+          = ClassMethods.getMethods(otype,
                                     Compilation.mangleName(mname),
                                     '\0', caller, language);
         if (methods != null && methods.length > 0)
           {
             nexp.methods = methods;
+            nexp.otype = otype;
+            nexp.mname = mname;
             return nexp.setProcedureKind('S');
           }
         ApplyExp aexp = new ApplyExp(SlotGet.staticField, args);
@@ -215,7 +218,7 @@ public class CompileNamedPart
                 Class cl = ClassType.getContextClass(name);
                 clas = QuoteExp.getInstance(Type.make(cl));
               }
-            catch (Throwable ex)
+            catch (Exception ex)
               {
               }
           }
@@ -378,6 +381,8 @@ class GetNamedExp extends ApplyExp
    */
   char kind;
   PrimProcedure[] methods;
+  ObjectType otype;
+  String mname;
 
   public String combinedName;
 
@@ -424,6 +429,7 @@ class GetNamedExp extends ApplyExp
     Expression context = pargs[0];
     Expression[] args = exp.getArgs();
     Expression[] xargs;
+    int adjust;
     switch (kind)
       {
       case 'M':
@@ -432,12 +438,14 @@ class GetNamedExp extends ApplyExp
         xargs[0] = pargs[0];
         xargs[1] = pargs[1];
         System.arraycopy(args, 0, xargs, 2, args.length);
+        adjust = 2;
         break;
       case 'N': // new
         decl = makeDecl;
         xargs = new Expression[args.length+1];
         System.arraycopy(args, 0, xargs, 1, args.length);
         xargs[0] = context;
+        adjust = 1;
         break;
       case 'I': // instance-of
         decl = instanceOfDecl;
@@ -445,13 +453,14 @@ class GetNamedExp extends ApplyExp
         System.arraycopy(args, 1, xargs, 2, args.length-1);
         xargs[0] = args[0];
         xargs[1] = context;
+        adjust = exp.firstSpliceArg > 0 ? 1 : 0;
         break;
       case 'C': // cast
         decl = castDecl;
         xargs = new Expression[args.length+1];
-        System.arraycopy(args, 1, xargs, 2, args.length-1);
+        System.arraycopy(args, 0, xargs, 1, args.length);
         xargs[0] = context;
-        xargs[1] = args[0];
+        adjust = 1;
         break;
       case 'S': // invoke-static
         decl = invokeStaticDecl;
@@ -459,12 +468,22 @@ class GetNamedExp extends ApplyExp
         xargs[0] = context;
         xargs[1] = pargs[1];
         System.arraycopy(args, 0, xargs, 2, args.length);
+        adjust = 2;
         break;
       default:
         return exp;
       }
     ApplyExp result = new ApplyExp(new ReferenceExp(decl), xargs);
+    if (exp.firstSpliceArg >= 0)
+      result.firstSpliceArg = exp.firstSpliceArg + adjust;
     result.setLine(exp);
+    if (methods != null && kind == 'S') {
+        return CompileInvoke.validateNamedInvoke(result, visitor,
+                                                 otype, mname,
+                                                 methods,
+                                                 Invoke.invokeStatic,
+                                                 required);
+    }
     return visitor.visit(result, required);
   }
 
