@@ -31,10 +31,10 @@ public class GetNamedPart extends Procedure2 implements HasSetter
     if (container instanceof Values)
       {
         Object[] values = ((Values) container).getValues();
-        Values result = new Values();
+        Values.FromTreeList result = new Values.FromTreeList();
         for (int i = 0;  i < values.length;  i++)
           {
-            Values.writeValues(apply2(values[i], part), result);
+            result.writeObject(apply2(values[i], part));
           }
         return result.canonicalize();
       }
@@ -47,7 +47,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter
   }
 
   public static Object getTypePart (Type type, String name)
-    throws Throwable
+    throws Exception
   {
     if (name.equals(CLASSTYPE_FOR))
       return type;
@@ -72,7 +72,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter
           {
             return gnu.kawa.reflect.SlotGet.staticField(type, name);
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             // FIXME!
           }
@@ -82,7 +82,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter
   }
 
   public static Object getNamedPart (Object container, Symbol part)
-    throws Throwable
+    throws Exception
   {
     String name = part.getName();
     container = Promise.force(container);
@@ -97,7 +97,7 @@ public class GetNamedPart extends Procedure2 implements HasSetter
             String pname = ((Package) container).getName();
             return ClassType.getContextClass(pname + '.' + name);
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
           }
       }
@@ -107,13 +107,13 @@ public class GetNamedPart extends Procedure2 implements HasSetter
   }
 
   public static Object getMemberPart(Object container, String name)
-    throws Throwable
+    throws Exception
   {
     try
       {
         return gnu.kawa.reflect.SlotGet.field(container, name);
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
         // FIXME!
       }
