@@ -3,10 +3,15 @@ import gnu.bytecode.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.math.*;
-import gnu.text.*;
+import gnu.kawa.io.FilePath;
+import gnu.kawa.io.Path;
+import gnu.kawa.io.URIPath;
 import gnu.kawa.functions.Arithmetic;
+import gnu.kawa.functions.LProcess;
 import gnu.kawa.reflect.Invoke;
 import gnu.kawa.reflect.LazyType;
+import gnu.lists.Blob;
+import gnu.lists.U8Vector;
 import java.util.*;
 
 /** A wrapper around a class type.
@@ -14,7 +19,7 @@ import java.util.*;
  * but may have a custom (language-specific) coercion method,
  * constructor, and name. */
 
-public class LangObjType extends ObjectType implements TypeValue
+public class LangObjType extends SpecialObjectType implements TypeValue
 {
   final int typeCode;
   private static final int PATH_TYPE_CODE = 1;
@@ -47,13 +52,13 @@ public class LangObjType extends ObjectType implements TypeValue
   private static final int PROMISE_TYPE_CODE = 28;
 
   public static final LangObjType pathType =
-    new LangObjType("path", "gnu.text.Path",
+    new LangObjType("path", "gnu.kawa.io.Path",
                     PATH_TYPE_CODE);
   public static final LangObjType filepathType =
-    new LangObjType("filepath", "gnu.text.FilePath",
+    new LangObjType("filepath", "gnu.kawa.io.FilePath",
                     FILEPATH_TYPE_CODE);
   public static final LangObjType URIType =
-    new LangObjType("URI", "gnu.text.URIPath",
+    new LangObjType("URI", "gnu.kawa.io.URIPath",
                     URI_TYPE_CODE);
 
   public static final LangObjType typeClass =
@@ -157,15 +162,15 @@ public class LangObjType extends ObjectType implements TypeValue
     new LangObjType("promise", "gnu.mapping.Lazy",
                     PROMISE_TYPE_CODE);
 
-  LangObjType(String name, String implClass, int typeCode)
-  {
-    super(name);
-    this.implementationType = ClassType.make(implClass);
-    this.typeCode = typeCode;
-    this.setSignature(this.implementationType.getSignature());
-  }
+    LangObjType(String name, String implClass, int typeCode) {
+        super(name, ClassType.make(implClass));
+        this.typeCode = typeCode;
+    }
 
-  ClassType implementationType;
+    @Override
+    public int isCompatibleWithValue(Type valueType) {
+        return getImplementationType().isCompatibleWithValue(valueType);
+    }
 
   public int compare(Type other)
   {
@@ -198,48 +203,7 @@ public class LangObjType extends ObjectType implements TypeValue
           return 1;
         break;
       }
-    return getImplementationType().compare(other.getImplementationType());
-  }
-
-  public Field getField(String name, int mask)
-  {
-    return implementationType.getField(name, mask);
-  }
-
-  public Method getMethod(String name, Type[] arg_types)
-  {
-    return implementationType.getMethod(name, arg_types);
-  }
-
-  public Method getDeclaredMethod(String name, int argCount)
-  {
-    return implementationType.getDeclaredMethod(name, argCount);
-  }
-
-  public int getMethods (Filter filter, int searchSupers,
-                         /* #ifdef JAVA5 */
-                         List<Method>
-                         /* #else */
-                         // Vector
-                         /* #endif */
-                         result)
-  {
-    return implementationType.getMethods(filter, searchSupers, result);
-  }
-
-  public java.lang.Class getReflectClass()
-  {
-    return implementationType.getReflectClass();
-  }
-
-  public Type getRealType()
-  {
-    return implementationType;
-  }
-
-  public Type getImplementationType()
-  {
-    return implementationType;
+    return getImplementationType().compare(other);
   }
 
   public void emitIsInstance(Variable incoming,
@@ -417,6 +381,14 @@ public class LangObjType extends ObjectType implements TypeValue
     return coerced;  
   }
 
+    public static U8Vector coerceToU8Vector(Object obj) {
+        if (obj instanceof LProcess)
+            return ((LProcess) obj).getValue().asPlainBytevector();
+        if (obj instanceof Blob)
+            return ((Blob) obj).asPlainBytevector();
+        return (U8Vector) obj;
+    }
+
   Method coercionMethod ()
   {
     switch (typeCode)
@@ -439,10 +411,11 @@ public class LangObjType extends ObjectType implements TypeValue
         return typeLangObjType.getDeclaredMethod("coerceIntNum", 1);
       case DFLONUM_TYPE_CODE:
         return typeLangObjType.getDeclaredMethod("coerceDFloNum", 1);
+      case U8VECTOR_TYPE_CODE:
+        return typeLangObjType.getDeclaredMethod("coerceToU8Vector", 1);
       case VECTOR_TYPE_CODE:
       case CONST_VECTOR_TYPE_CODE:
       case S8VECTOR_TYPE_CODE:
-      case U8VECTOR_TYPE_CODE:
       case S16VECTOR_TYPE_CODE:
       case U16VECTOR_TYPE_CODE:
       case S32VECTOR_TYPE_CODE:
@@ -682,7 +655,6 @@ public class LangObjType extends ObjectType implements TypeValue
       case CONST_VECTOR_TYPE_CODE:
       case VECTOR_TYPE_CODE:
       case S8VECTOR_TYPE_CODE:
-      case U8VECTOR_TYPE_CODE:
       case S16VECTOR_TYPE_CODE:
       case U16VECTOR_TYPE_CODE:
       case S32VECTOR_TYPE_CODE:
@@ -694,6 +666,7 @@ public class LangObjType extends ObjectType implements TypeValue
       case STRING_TYPE_CODE:
       case LIST_TYPE_CODE:
       case REGEX_TYPE_CODE:
+      case PROMISE_TYPE_CODE:
         code.emitCheckcast(implementationType);
         break;
       default:
@@ -721,11 +694,11 @@ public class LangObjType extends ObjectType implements TypeValue
     switch (typeCode)
       {
       case PATH_TYPE_CODE:
-        return new PrimProcedure("gnu.text.Path", "valueOf", 1);
+        return new PrimProcedure("gnu.kawa.io.Path", "valueOf", 1);
       case FILEPATH_TYPE_CODE:
-        return new PrimProcedure("gnu.text.FilePath", "makeFilePath", 1);
+        return new PrimProcedure("gnu.kawa.io.FilePath", "makeFilePath", 1);
       case URI_TYPE_CODE:
-        return new PrimProcedure("gnu.text.URIPath", "makeURI", 1);
+        return new PrimProcedure("gnu.kawa.io.URIPath", "makeURI", 1);
       case VECTOR_TYPE_CODE:
         return new PrimProcedure("gnu.lists.FVector", "make", 1);
       case U8VECTOR_TYPE_CODE:
@@ -741,6 +714,10 @@ public class LangObjType extends ObjectType implements TypeValue
       }
   }
 
+    /* #ifndef JAVA8 */
+    public String encodeType(Language language) { return null; }
+    /* #endif */
+
   public static final ClassType typeLangObjType =
     ClassType.make("gnu.kawa.lispexpr.LangObjType");
 }
