@@ -23,6 +23,8 @@ public abstract class LispLanguage extends Language
   static public final String unquote_str = "unquote";
   static public final String unquotesplicing_str = "unquote-splicing";
   static public final String quasiquote_str = "quasiquote";
+    public static final Symbol quasiquote_sym =
+        Namespace.EmptyNamespace.getSymbol(quasiquote_str);
   static public final String splice_str = "$splice$";
   static public final Symbol splice_sym = Namespace.EmptyNamespace.getSymbol(splice_str);
   /** Used for Kawa infix ':' operator. */
@@ -135,7 +137,18 @@ public abstract class LispLanguage extends Language
   public void resolve (Compilation comp)
   {
     Translator tr = (Translator) comp;
-    tr.resolveModule(tr.getModule());
+    ModuleExp mexp = tr.getModule();
+    tr.resolveModule(mexp);
+    if (tr.subModuleMap != null) {
+        String mainName = tr.mainClass.getName();
+        ModuleInfo subinfo = tr.subModuleMap.get(mainName);
+        if (subinfo != null
+            && ! (mexp.body == QuoteExp.voidExp && mexp.firstDecl() == null)) {
+            ModuleExp submodule = subinfo.getModuleExpRaw();
+            tr.error('e', "module has both statements and a submodule with the same name: "+tr.mainClass.getName(),
+                     submodule != null ? submodule : mexp);
+        }
+    }
   }
 
   public Declaration declFromField (ModuleExp mod, Object fvalue, Field fld)
