@@ -161,14 +161,14 @@ public class ClassMethods extends Procedure2
    *          + (number of possibly applicable methods.
    */
   public static long selectApplicable(PrimProcedure[] methods,
-                                      Type[] atypes)
+                                      Type[] atypes, Type restType)
   {
     int limit = methods.length;
     int numDefApplicable = 0;
     int numPosApplicable = 0;
     for (int i = 0;  i < limit;  )
       {
-        int code = methods[i].isApplicable(atypes);
+        int code = methods[i].isApplicable(atypes, restType);
         if (code < 0)
           { // Definitely not applicable.
             // swap(methods[limit-1], methods[i]):
@@ -199,7 +199,8 @@ public class ClassMethods extends Procedure2
    * @return number of methods that apply, NO_MATCH_TOO_FEW_ARGS,
    *  or NO_MATCH_TOO_MANY_ARGS.
    */
-  public static int selectApplicable(PrimProcedure[] methods, int numArgs)
+  public static int selectApplicable(PrimProcedure[] methods,
+                                     int numArgs, boolean maybeMore)
   {
     int limit = methods.length;
     int numTooManyArgs = 0;
@@ -211,7 +212,7 @@ public class ClassMethods extends Procedure2
         int min = Procedure.minArgs(num);
         int max = Procedure.maxArgs(num);
         boolean ok = false;
-        if (numArgs < min)
+        if (numArgs < min && !maybeMore)
           numTooFewArgs++;
         else if (numArgs > max && max >= 0)
           numTooManyArgs++;
