@@ -92,7 +92,7 @@ public class CompileBuildObject {
             try {
                 builder = (CompileBuildObject) Class.forName(builderName).newInstance();
             }
-            catch (Throwable ex) {
+            catch (Exception ex) {
                 comp.error('w', "while creating JavafxObjectBuilder for "+ctype+" - caught "+ex);
                 builder = new CompileBuildObject();
             }
@@ -134,7 +134,7 @@ public class CompileBuildObject {
     }
 
     public boolean hasAddChildMethod() {
-        return ClassMethods.selectApplicable(ClassMethods.getMethods(ctype, "add", 'V', null, getLanguage()), 2) > 0;
+        return ClassMethods.selectApplicable(ClassMethods.getMethods(ctype, "add", 'V', null, getLanguage()), 2, false/*??*/) > 0;
     }
 
     public Member findNamedMember (String name) {
