@@ -195,7 +195,7 @@ public class FieldLocation<T> extends ClassMemberLocation<T>
       {
         setup();
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
         if (throwIfUnbound) throw new UnboundLocationException(this); 
         return defaultValue;
@@ -241,9 +241,9 @@ public class FieldLocation<T> extends ClassMemberLocation<T>
       {
         return (T) rfield.get(instance);
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
-	throw WrappedException.wrapIfNeeded(ex);
+	throw WrappedException.rethrow(ex);
       }
   }
 
@@ -256,7 +256,7 @@ public class FieldLocation<T> extends ClassMemberLocation<T>
 	  {
 	    rfield.set(instance, newValue);
 	  }
-	catch (Throwable ex)
+	catch (Exception ex)
 	  {
 	    throw WrappedException.wrapIfNeeded(ex);
 	  }
@@ -322,7 +322,7 @@ public class FieldLocation<T> extends ClassMemberLocation<T>
               {
                 setup();
               }
-            catch (Throwable ex)
+            catch (Exception ex)
               {
                 return false;
               }
@@ -350,7 +350,7 @@ public class FieldLocation<T> extends ClassMemberLocation<T>
           {
             setup();
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             return false;
           }
