@@ -1,12 +1,27 @@
 (define as-xml (make <gnu.kawa.xml.OutputAsXML>))
 
-(define-constant comment <gnu.kawa.xml.KComment>)
+(define-alias comment gnu.kawa.xml.KComment)
 
-(define-constant processing-instruction <gnu.kawa.xml.KProcessingInstruction>)
+; http://download.plt-scheme.org/doc/html/xml/index.html#%28def._%28%28lib._xml/main..ss%29._element-name%29%29 has these:
+; element-name
+; element-attributes
+; element-content
+; atribute-name
+; atribute-value
+; p-i-target-name
+; p-i-instruction
+
+(define-alias processing-instruction gnu.kawa.xml.KProcessingInstruction)
 
 ;;; Parse an XML file (specified by a URL or url string), giving a <document>.
-(define (parse-xml-from-url url) :: <document>
-  (invoke-static <gnu.kawa.xml.Document> 'parse url))
+(define (parse-xml-from-url url) :: document
+  (gnu.kawa.xml.Document:parse url))
+
+(define (element-name element::gnu.kawa.xml.KElement) ::symbol
+  (element:getNodeSymbol))
+
+(define (attribute-name attr::gnu.kawa.xml.KAttr) ::symbol
+  (attr:getNodeSymbol))
 
 #|
 (define (parse-nsxml-from-url url) :: <document>
