@@ -231,12 +231,13 @@
   (if (not (substring-spec-ok? s start end))
       (error "Illegal substring spec." proc s start end)))
 
+#|
 (define-private (%check-substring-spec proc (s :: string) (start :: int) (end :: int))
   (if (or (> 0 start)
 	  (> start end)
 	  (> end (string-length s)))
       (error "Illegal substring spec." proc s start end)))
-
+|#
 
 ;;; Defined by R5RS, so commented out here.
 ;(define (string . chars)
@@ -277,9 +278,12 @@
   (if (and (zero? start) (= end (string-length s))) s
       (substring s start end)))
 
+#|
 (define (string-copy s . maybe-start+end)
   (let-string-start+end (start end) string-copy s maybe-start+end
     (substring s start end)))
+|#
+(define-alias string-copy kawa.lib.strings:string-copy)
 
 ;This library uses the R5RS SUBSTRING, but doesn't export it.
 ;Here is a definition, just for completeness.
@@ -1311,13 +1315,19 @@
 ;;; string-copy! to tstart from [fstart fend]
 ;;; 	Guaranteed to work, even if s1 eq s2.
 
+(define-alias string-fill! kawa.lib.strings:string-fill!)
+#|
 (define (string-fill! s char . maybe-start+end)
   (check-arg char? char string-fill!)
   (let-string-start+end (start end) string-fill! s maybe-start+end
     (do ((i (- end 1) (- i 1)))
 	((< i start))
       (string-set! s i char))))
+|#
 
+(define-alias string-copy! kawa.lib.strings:string-copy!)
+(define-alias %string-copy! kawa.lib.strings:string-copy!)
+#|
 (define (string-copy! to (tstart :: int) (from :: string)
 		      #!optional (fstart :: int 0)
 		      (fend :: int (from:length)))
@@ -1340,6 +1350,7 @@
 	    ((< i fstart))
 	  (string-set! to j (string-ref from i))
 	  (set! j (- j 1))))))
+|#
 
 ;;; Returns starting-position in STRING or #f if not true.
 ;;; This implementation is slow & simple. It is useful as a "spec" or for
@@ -1574,13 +1585,14 @@
 
 ;(define (string->list s . maybe-start+end)
 ;  (apply string-fold-right cons '() s maybe-start+end))
-
+(define-alias string->list kawa.lib.strings:string->list)
+#|
 (define (string->list s . maybe-start+end)
   (let-string-start+end (start end) string->list s maybe-start+end
     (do ((i (- end 1) (- i 1))
 	 (ans '() (cons (string-ref s i) ans)))
 	((< i start) ans))))
-
+|#
 ;;; Defined by R5RS, so commented out here.
 ;(define (list->string lis) (string-unfold null? car cdr lis))
 
