@@ -111,7 +111,7 @@ class ProcActionListener implements ActionListener
       }
     catch (Throwable ex)
       {
-	throw new gnu.mapping.WrappedException(ex);
+        gnu.mapping.WrappedException.rethrow(ex);
       }
   }
 }
