@@ -1,7 +1,9 @@
 package gnu.lists;
 import gnu.mapping.*;
+import gnu.kawa.io.CharArrayOutPort;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
 import java.text.FieldPosition;
-import gnu.text.Path;
 
 public abstract class AbstractFormat extends java.text.Format
 {
