@@ -119,7 +119,7 @@ public abstract class AbstractSequence<E>
     int i = 0;
     for (int iter = startPos();  (iter = nextPos(iter)) != 0;  i++)
       {
-	Object v = getPosPrevious(iter);
+        Object v = getPosPrevious(iter);
         if (o==null ? v==null : o.equals(v))
 	  {
 	    releasePos(iter);
@@ -191,19 +191,19 @@ public abstract class AbstractSequence<E>
     return getIterator();
   }
 
-  public final SeqPosition<E> getIterator()
+  public final SeqPosition<E, AbstractSequence<E>> getIterator()
   {
     return getIterator(0);
   }
 
-  public SeqPosition<E> getIterator(int index)
+  public SeqPosition<E, AbstractSequence<E>> getIterator(int index)
   {
-    return new SeqPosition<E>(this, index, false);
+      return new SeqPosition<E,AbstractSequence<E>>(this, index, false);
   }
 
-  public SeqPosition<E> getIteratorAtPos(int ipos)
+  public SeqPosition<E, AbstractSequence<E>> getIteratorAtPos(int ipos)
   {
-    return new SeqPosition<E>(this, copyPos(ipos));
+    return new SeqPosition<E,AbstractSequence<E>>(this, copyPos(ipos));
   }
 
   /* #ifdef JAVA2 */
@@ -325,10 +325,10 @@ public abstract class AbstractSequence<E>
     if (index < 0 || index >= size())
       throw new IndexOutOfBoundsException();
     int ipos = createPos(index, false);
-    Object result = getPosNext(ipos);
+    E result = (E) getPosNext(ipos);
     removePos(ipos, 1);
     releasePos(ipos);
-      return (E) result;
+    return result;
   }
 
   public boolean remove(Object o)
@@ -393,7 +393,9 @@ public abstract class AbstractSequence<E>
    * @param isAfter should the position have the isAfter property
    * @exception IndexOutOfBoundsException if index is out of bounds
    */
-  public abstract int createPos (int index, boolean isAfter);
+  public int createPos (int index, boolean isAfter) {
+      return (index << 1) | (isAfter ? 1 : 0);
+  }
 
   public int createRelativePos(int pos, int delta, boolean isAfter)
   {
@@ -435,10 +437,7 @@ public abstract class AbstractSequence<E>
   /**
    * Get the offset from the beginning corresponding to a position cookie.
    */
-  protected int nextIndex(int ipos)
-  {
-    return getIndexDifference(ipos, startPos());
-  }
+  protected abstract int nextIndex(int ipos);
 
   protected int fromEndIndex(int ipos)
   {
@@ -467,7 +466,8 @@ public abstract class AbstractSequence<E>
 
   public String getNextTypeName(int ipos)
   {
-    return null;
+    Object type = getNextTypeObject(ipos);
+    return type == null ? null : type.toString();
   }
 
   public E getNextTypeObject(int ipos)
@@ -588,7 +588,9 @@ public abstract class AbstractSequence<E>
   /** Get the element following the specified position.
    * @param ipos the specified position.
    * @return the following element, or eofValue if there is none.
-   * Called by SeqPosition.getNext. */
+   * Called by SeqPosition.getNext.
+   * FIXME Should change eof handling so return type can be E.
+   */
   public Object getPosNext(int ipos)
   {
     if (! hasNext(ipos))
@@ -598,7 +600,9 @@ public abstract class AbstractSequence<E>
 
   /** Get the element before the specified position.
    * @param ipos the specified position.
-   * @return the following element, or eofValue if there is none. */
+   * @return the following element, or eofValue if there is none.
+   * FIXME Should change eof handling so return type can be E.
+   */
   public Object getPosPrevious(int ipos)
   {
     int index = nextIndex(ipos);
@@ -764,13 +768,13 @@ public abstract class AbstractSequence<E>
     return subSequencePos(start.ipos, end.ipos);
   }
 
-  protected Sequence subSequencePos(int ipos0, int ipos1)
+  protected Sequence<E> subSequencePos(int ipos0, int ipos1)
   {
-    return new SubSequence(this, ipos0, ipos1);
+    return new SubSequence<E>(this, ipos0, ipos1);
   }
 
   /* #ifdef JAVA2 */
-  public List subList(int fromIx, int toIx)
+  public List<E> subList(int fromIx, int toIx)
   {
     return subSequencePos(createPos(fromIx, false),
                           createPos(toIx, true));
@@ -796,12 +800,20 @@ public abstract class AbstractSequence<E>
     while (! equals(it, iposEnd))
       {
 	if (! hasNext(it))
-	  throw new RuntimeException();
+          throw new RuntimeException();
 	out.writeObject(getPosNext(it));
         it = nextPos(it);
       }
     releasePos(it);
   }
+  
+    public void consume(int fromIndex, int toIndex, Consumer out) {
+        int ipos0 = createPos(fromIndex, false);
+        int ipos1 = createPos(toIndex, true);
+        consumePosRange(ipos0, ipos1, out);
+        releasePos(ipos0);
+        releasePos(ipos1);
+    }
 
   public void consume(Consumer out)
   {
