@@ -91,11 +91,10 @@ public class S32Vector extends SimpleVector
     return Convert.toObject(data[index]);
   }
 
-  public Object setBuffer(int index, Object value)
+  @Override
+  public void setBuffer(int index, Object value)
   {
-    int old = data[index];
     data[index] = Convert.toInt(value);
-    return Convert.toObject(old);
   }
 
   public final void setIntAt(int index, int value)
@@ -123,15 +122,6 @@ public class S32Vector extends SimpleVector
 
   public String getTag() { return "s32"; }
 
-  public boolean consumeNext (int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeInt(data[index]);
-    return true;
-  }
-
   public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
