@@ -96,11 +96,10 @@ public class S64Vector extends SimpleVector
     return (int) data[index];
   }
 
-  public Object setBuffer(int index, Object value)
+  @Override
+  public void setBuffer(int index, Object value)
   {
-    long old = data[index];
     data[index] = Convert.toLong(value);
-    return Convert.toObject(old);
   }
 
   public final void setLongAt(int index, long value)
@@ -128,15 +127,6 @@ public class S64Vector extends SimpleVector
 
   public String getTag() { return "s64"; }
 
-  public boolean consumeNext (int ipos, Consumer out)
-  {
-    int index = ipos >>> 1;
-    if (index >= size)
-      return false;
-    out.writeLong(data[index]);
-    return true;
-  }
-
   public void consumePosRange (int iposStart, int iposEnd, Consumer out)
   {
     if (out.ignoring())
