@@ -81,11 +81,10 @@ public class U8Vector extends ByteVector
     return Convert.toObjectUnsigned(data[index]);
   }
 
-  public Object setBuffer(int index, Object value)
+  @Override
+  public void setBuffer(int index, Object value)
   {
-    byte old = data[index];
     data[index] = Convert.toByteUnsigned(value);
-    return Convert.toObjectUnsigned(old);
   }
 
   public int getElementKind()
@@ -104,13 +103,13 @@ public class U8Vector extends ByteVector
     public String toUtf8(int start, int length) {
       if (start+length>size) throw new IndexOutOfBoundsException();
       /* #ifdef JAVA7 */  
-      // return new String(data, start, length, java.nio.charset.StandardCharsets.UTF_8);
+      return new String(data, start, length, java.nio.charset.StandardCharsets.UTF_8);
       /* #else */
-      try {
-        return new String(data, start, length, "UTF-8");
-      } catch (UnsupportedEncodingException ex) {
-          throw new RuntimeException(ex);
-      }
+      // try {
+      //   return new String(data, start, length, "UTF-8");
+      // } catch (UnsupportedEncodingException ex) {
+      //     throw new RuntimeException(ex);
+      // }
       /* #endif */
   }
 }
