@@ -24,19 +24,12 @@ public class TextUtils
     StringBuffer sbuf = new StringBuffer(100);
     if (node instanceof Values)
       {
-	TreeList tlist = (TreeList) node;
+	Values vals = (Values) node;
 	int index = 0;
-	for (;;)
-	  {
-	    int kind = tlist.getNextKind(index);
-	    if (kind == Sequence.EOF_VALUE)
-	      break;
-	    if (kind == Sequence.OBJECT_VALUE)
-	      stringValue(tlist.getPosNext(index), sbuf);
-	    else
-	      tlist.stringValue(tlist.posToDataIndex(index), sbuf);
-	    index = tlist.nextPos(index);
-	  }
+        for (int ipos = 0; (ipos = vals.nextPos(ipos)) != 0; )
+          {
+            stringValue(vals.getPosPrevious(ipos), sbuf);
+          }
       }
     else
       stringValue(node, sbuf);
@@ -76,12 +69,13 @@ public class TextUtils
         StringBuffer sbuf = new StringBuffer();
         if (arg instanceof Values)
           {
-            Object[] vals = ((Values) arg).getValues();
-            for (int i = 0;  i < vals.length; i++)
+            Values vals = (Values) arg;
+            int count = -1;
+            for (int ipos = 0; (ipos = vals.nextPos(ipos)) != 0; )
               {
-                if (i > 0)
+                if (++count > 0)
                   sbuf.append(' ');
-                stringValue(vals[i], sbuf);
+                stringValue(vals.getPosPrevious(ipos), sbuf);
               }
           }
         else
