@@ -13,6 +13,9 @@ import gnu.text.SourceMessages;
 import java.io.Reader;
 import java.util.Vector;
 import gnu.kawa.functions.ConstantFunction0;
+import gnu.kawa.io.CharArrayInPort;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.OutPort;
 import gnu.kawa.reflect.ClassMethods;
 import gnu.math.IntNum;
 import gnu.kawa.xml.*;
@@ -888,7 +891,7 @@ public class XQuery extends Language
           return XDataType.decimalType;
         if (name.equals("gnu.math.Duration"))
           return XDataType.durationType;
-        if (name.equals("gnu.text.Path"))
+        if (name.equals("gnu.kawa.io.Path"))
           return  XDataType.anyURIType;
       }
     return Type.make(clas);
