@@ -4,13 +4,13 @@
 package gnu.xquery.util;
 import gnu.mapping.*;
 import gnu.xml.*;
+import gnu.kawa.io.Path;
 import gnu.kawa.xml.*;
 import gnu.lists.*;
 import java.util.Stack;
 import java.net.*;
 import gnu.bytecode.ClassType;
 import gnu.xquery.lang.XQuery;
-import gnu.text.Path;
 
 public class NodeUtils
 {
@@ -403,7 +403,7 @@ public class NodeUtils
           {
             throw new RuntimeException("invalid collection-resolver: class "+cname+" not found");
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             throw new RuntimeException("invalid collection-resolver: "+ex);
           }
@@ -461,7 +461,7 @@ public class NodeUtils
         Document.parseCached(uri);
         return true;
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
         return false;
       }
