@@ -1,5 +1,6 @@
 package kawa;
 
+import gnu.kawa.io.InPort;
 import gnu.kawa.io.OutPort;
 import gnu.mapping.*;
 import gnu.expr.Language;
@@ -26,7 +27,7 @@ public class GuiConsole extends JFrame
   ReplDocument document;
 
   public static void main(String[] args) {
-    repl.noConsole = false;
+    InPort.noConsole = false;
     int iArg = repl.processArgs(args, 0, args.length);
     repl.getLanguage();
     repl.setArgs(args, iArg);
