@@ -3,10 +3,20 @@ package kawa;
 import gnu.mapping.*;
 import gnu.expr.*;
 import java.io.*;
-import gnu.text.*;
 import gnu.lists.*;
 import gnu.bytecode.ArrayClassLoader;
 import gnu.bytecode.ZipLoader;
+import gnu.kawa.io.BinaryInPort;
+import gnu.kawa.io.FilePath;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.NBufferedInputStream;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
+import gnu.kawa.io.TtyInPort;
+import gnu.kawa.util.ExitCalled;
+import gnu.text.Lexer;
+import gnu.text.SourceMessages;
+import gnu.text.SyntaxException;
 import java.net.URL;
 
 /** Utility functions (static methods) for kawa.repl.
@@ -25,9 +35,8 @@ public class Shell
   private static Class[] noClasses = { };
   private static  Class[] boolClasses = { Boolean.TYPE };
   private static  Class[] xmlPrinterClasses
-    = {gnu.mapping.OutPort.class,  java.lang.Object.class };
-  private static  Class[] httpPrinterClasses
-    = {gnu.mapping.OutPort.class };
+    = {OutPort.class, Object.class };
+  private static  Class[] httpPrinterClasses = {OutPort.class };
   private static Object portArg = "(port)";
 
   /** A table of names of known output formats.
@@ -150,7 +159,7 @@ public class Shell
 	else
 	  return (Consumer) format;
       }
-    catch (Throwable ex)
+    catch (Exception ex)
       {
 	throw new RuntimeException("cannot get output-format '"
 				   + defaultFormatName + "' - caught " + ex);
@@ -242,6 +251,7 @@ public class Shell
       {
         // Nothing - we'll just lose some minor functionality.
       }
+    Environment saveEnv = Environment.setSaveCurrent(env);
     try
       {
 	for (;;)
@@ -259,8 +269,10 @@ public class Shell
                   sawError = false;
 		if (comp == null) // ??? end-of-file
 		  break;
-		if (sawError)
-		  continue;
+		if (sawError) {
+                    comp.lexical.pop(comp.mainLambda);
+                    continue;
+                }
 
 		// Skip whitespace, in case (read-char) or similar is called:
 		int ch;
@@ -283,7 +295,11 @@ public class Shell
 		if (ch < 0)
 		  break;
 	      }
-	    catch (Throwable e)
+            catch (Error e)
+              {
+                throw e;
+              }
+            catch (Throwable e)
 	      {
 		if (! interactive)
 		  return e;
@@ -293,6 +309,7 @@ public class Shell
       }
     finally
       {
+        Environment.restoreCurrent(saveEnv);
 	if (out != null)
 	  ctx.consumer = saveConsumer;
         Language.restoreCurrent(saveLanguage);
@@ -383,6 +400,16 @@ public class Shell
       }
   }
 
+    static InPort openFile(InputStream fs, Path path) throws IOException {
+        Object conv = Environment.user().get("port-char-encoding");
+        InPort src;
+        if (conv == null || conv == Boolean.TRUE)
+            return BinaryInPort.openHeuristicFile(fs, path);
+        else
+            return InPort.openFile(fs, path, conv);
+    }
+
+
   /** Run a named source file, compiled .zip, or class.
    * We try in order if {@code fname} names a compiled zip file,
    * or names some other file (in which case it is assumed to be source),
@@ -419,12 +446,20 @@ public class Shell
             Environment env = Environment.getCurrent();
             return runFile(fs, path, env, lineByLine, skipLines);
           }
+        catch (Error e)
+          {
+            throw e;
+          }
         catch (Throwable e)
           {
             e.printStackTrace(System.err);
             return false;
           }
       }
+    catch (Error e)
+      {
+        throw e;
+      }
     catch (Throwable e)
       {
         Class clas;
@@ -432,7 +467,7 @@ public class Shell
           {
             clas = Class.forName(fname);
           }
-        catch (Throwable ex)
+        catch (Exception ex)
           {
             System.err.println("Cannot read file "+e.getMessage());
             return false;
@@ -442,8 +477,13 @@ public class Shell
             runClass(clas, Environment.getCurrent());
             return true;
           }
+        catch (Error ex)
+          {
+            throw ex;
+          }
         catch (Throwable ex)
           {
+              //ExitCalled.check(e);
             ex.printStackTrace();
             return false;
           }
@@ -459,15 +499,16 @@ public class Shell
                                         Environment env,
                                         boolean lineByLine, int skipLines)
             throws Throwable {
-        if (! (fs instanceof BufferedInputStream))
-            fs = new BufferedInputStream(fs);
+        if (! (fs instanceof BufferedInputStream)
+            && ! (fs instanceof NBufferedInputStream))
+            fs = new NBufferedInputStream(fs);
         Language language = Language.getDefaultLanguage();
         Path savePath = (Path) currentLoadPath.get();
         try {
             currentLoadPath.set(path);
             CompiledModule cmodule = checkCompiledZip(fs, path, env, language);
             if (cmodule == null) {
-                InPort src = InPort.openFile(fs, path);
+                InPort src = openFile(fs, path);
                 while (--skipLines >= 0)
                     src.skipRestOfLine();
                 try {
@@ -522,6 +563,9 @@ public class Shell
                 return null;
             return new CompiledModule(comp.getModule(), inst, language);
         }
+        catch (Error ex) {
+            throw ex;
+        }
         catch (Throwable ex) {
             if (! (ex instanceof SyntaxException)
                 || ((SyntaxException) ex).getMessages() != messages) {
