@@ -82,19 +82,18 @@ public class Eval
         Compilation saveComp = Compilation.setSaveCurrent(tr);
         try
           {
-            int first = tr.formStack.size();
             tr.scanBody(body, mod, false);
-            tr.firstForm = first;
             tr.finishModule(mod);
+            if (body instanceof PairWithPosition)
+                mod.setFile(((PairWithPosition) body).getFileName());
+            tr.setEvalName();
+            tr.process(Compilation.RESOLVED);
           }
         finally
           {
             Compilation.restoreCurrent(saveComp);
           }
 
-	if (body instanceof PairWithPosition)
-	  mod.setFile(((PairWithPosition) body).getFileName());
-	tr.setEvalName();
 	ModuleExp.evalModule(env, ctx, tr, null, null);
 	if (messages.seenErrors())
 	  throw new RuntimeException("invalid syntax in eval form:\n"
