@@ -26,8 +26,8 @@ public class ListPat extends Pattern
 	if (obj instanceof Pair)
 	  {
 	    Pair p = (Pair)obj;
-	    vars[start_vars + i] = p.car;
-	    obj = p.cdr;
+	    vars[start_vars + i] = p.getCar();
+	    obj = p.getCdr();
 	  }
 	else if (i < min)
 	  return false;
