@@ -47,7 +47,7 @@ public class PatternScope extends LetExp
 	newScope.patternNesting
 	  = new StringBuffer(oldScope.patternNesting.toString());
       }
-    newScope.outer = tr.currentScope();
+    newScope.setOuter(tr.currentScope());
     return newScope;
   }
 
