@@ -172,14 +172,14 @@ public class Record
     String mangledName = Compilation.mangleNameIfNeeded(name);
     ClassType clas = new ClassType(mangledName);
     clas.setSuper(superClass);
-    clas.access_flags = Access.PUBLIC;
+    clas.setModifiers(Access.PUBLIC|Access.SUPER);
 
     // Generate the (default) constructor.
     Method constructor = clas.addMethod ("<init>", Type.typeArray0,
-					  Type.void_type, Access.PUBLIC);
+					  Type.voidType, Access.PUBLIC);
     Method superConstructor
       = superClass.addMethod ("<init>", Type.typeArray0,
-			       Type.void_type, Access.PUBLIC);
+			       Type.voidType, Access.PUBLIC);
     gnu.bytecode.CodeAttr code = constructor.startCode();
     code.emitPushThis();
     code.emitInvokeSpecial(superConstructor);
@@ -198,12 +198,12 @@ public class Record
     while (fnames != LList.Empty)
       {
 	Pair pair = (Pair) fnames;
-	String fname = pair.car.toString();
+	String fname = pair.getCar().toString();
 	//fnamesBuf.append(fname);  fnamesBuf.append('\n');
 	fld = clas.addField(Compilation.mangleNameIfNeeded(fname),
 			    Type.pointer_type, Access.PUBLIC);
 	fld.setSourceName(fname.intern());
-	fnames = (LList) pair.cdr;
+	fnames = (LList) pair.getCdr();
       }
     /*
     fld = clas.addField("$FieldNames$", Compilation.typeString,
@@ -261,7 +261,7 @@ public class Record
       {
 	if ((field.getModifiers() & (Modifier.STATIC|Modifier.PUBLIC))
 	    == Modifier.PUBLIC)
-	  vec.addElement(field.getSourceName());
+	  vec.addElement(SimpleSymbol.valueOf(field.getSourceName()));
       }
     for (int i = vec.size();  --i >= 0; )
       {
