@@ -2,6 +2,9 @@
 (require <kawa.lib.std_syntax>)
 (require <kawa.lib.syntax>)
 
+(define (bytevector? x) ::boolean
+  (instance? x bytevector))
+
 (define (make-bytevector (n ::int) #!optional (init ::int 0)) :: bytevector
   (make gnu.lists.U8Vector n init))
 
@@ -69,4 +72,3 @@
    (((v:toString):substring start end):getBytes
     (cond-expand (java-7 java.nio.charset.StandardCharsets:UTF_8)
                  (else "UTF-8")))))
-
