@@ -1,5 +1,6 @@
 (require <kawa.lib.prim_syntax>)
 (require <kawa.lib.std_syntax>)
+(require <kawa.lib.case_syntax>)
 (require <kawa.lib.syntax>)
 (require <kawa.lib.strings>)
 (require <kawa.lib.ports>)
@@ -31,12 +32,9 @@
 (define (symbol->string (s <symbol>)) :: constant-string
   (s:toString))
 
-(define-procedure symbol=?
-  (lambda (s1::symbol s2::symbol)::boolean
-	  (gnu.mapping.Symbol:equals s1 s2))
-  (lambda (s1::symbol s2::symbol #!rest r)::boolean
-	  (and (gnu.mapping.Symbol:equals s1 s2)
-	       (apply symbol=? s2 r))))
+(define (symbol=? s1::symbol s2::symbol #!rest r)::boolean
+  (and (gnu.mapping.Symbol:equals s1 s2)
+       (or (null? r) (apply symbol=? s2 r))))
 
 (define (symbol-local-name s::symbol) ::constant-string
   (s:getLocalPart))
@@ -81,7 +79,9 @@
   (case version
     ((4) (static-field <kawa.standard.Scheme> 'r4Environment))
     ((5) (static-field <kawa.standard.Scheme> 'r5Environment))
-    (else (error "scheme-report-environment version must be 4 or 5"))))
+    (else (primitive-throw
+           (kawa.lang.NamedException:makeError
+            "scheme-report-environment version must be 4 or 5")))))
 
 (define (interaction-environment)
   (invoke-static <gnu.mapping.Environment> 'user))
@@ -105,6 +105,13 @@
    (thunk)
    (after)))
 
+(define (promise? obj) ::boolean
+  (instance? obj gnu.mapping.Lazy))
+
+(define (make-promise obj) ::gnu.mapping.Lazy
+  (if (gnu.mapping.Lazy? obj) obj
+      (gnu.mapping.Promise:makeBoundPromise obj)))
+
 (define (promise-set-value! promise::gnu.mapping.Promise value) ::void
   (promise:setValue value))
 
@@ -124,33 +131,11 @@
   (gnu.mapping.Promise:force arg))
 
 (define (eager value)
-  (if (gnu.mapping.Promise? value)
-      (gnu.mapping.Promise value: value)
-      value))
-
-(define (throw #!rest (args ::Object[])) ::never-returns
-  (let ((len args:length))
-    (if (> len 0)
-        (let ((key (args 0)))
-          (cond ((symbol? key)
-                 (primitive-throw (kawa.lang.NamedException key args)))
-                ((and (java.lang.Throwable? key) (= len 1))
-                 (gnu.kawa.reflect.Throw:doThrow key)))))
-    (primitive-throw (kawa.lang.GenericError "bad arguments to throw"))))
-              
-;;; The one-argument case is a standard DSSSL procedure.
-;;; The multi-argument extension matches Guile.
-(define (error msg . args)  ::never-returns
-  (set! msg (call-with-output-string (lambda (port) (display msg port))))
-  (set! args (map
-	      (lambda (arg)
-		(call-with-output-string (lambda (port) (write arg port))))
-	      args))
-  (apply throw 'misc-error msg args))
+  eager)
 
 (define (base-uri #!optional (node #!null))
   (let ((uri (if (eq? node #!null)
-		 (gnu.text.Path:currentPath)
+		 (gnu.kawa.io.Path:currentPath)
 		 ((as <gnu.kawa.xml.KNode> node):baseURI))))
     (if (eq? uri #!void) #f uri)))
 
