@@ -11,8 +11,7 @@
 (require <kawa.lib.reflection>)
 (require <kawa.lib.lists>)
 (require <kawa.lib.scheme.eval>)
-
-(import (only (kawa standard begin) begin))
+(require <kawa.lib.prim_imports>)
 
 (define-syntax defmacro
   (syntax-rules ()
