@@ -5,7 +5,10 @@ package kawa.standard;
 import kawa.lang.*;
 import gnu.expr.*;
 import gnu.lists.*;
+import gnu.kawa.io.FilePath;
+import gnu.kawa.io.Path;
 import gnu.mapping.*;
+import java.io.File;
 import gnu.bytecode.ObjectType;
 import java.util.*;
 import kawa.lang.Translator.FormStack;
@@ -19,7 +22,7 @@ public class ImportFromLibrary extends Syntax
 {
     public static final ImportFromLibrary instance = new ImportFromLibrary();
 
-    public String[] classPrefixPath = { "", "kawa.lib." };
+    public static String[] classPrefixPath = { "", "kawa.lib." };
 
     private static final String BUILTIN = "<builtin>";
     private static final String MISSING = null;
@@ -41,7 +44,7 @@ public class ImportFromLibrary extends Syntax
         { "21", "real-time-multithreading", MISSING },
         { "23", "error", BUILTIN },
         { "25", "multi-dimensional-arrays", BUILTIN },
-        { "26", "cut", BUILTIN },
+        { "26", "cut", "gnu.kawa.slib.cut" },
         { "27", "random-bits", MISSING },
         { "28", "basic-format-strings", BUILTIN },
         { "29", "localization", MISSING },
@@ -107,10 +110,11 @@ public class ImportFromLibrary extends Syntax
             } else
                 srfiName = demangled.substring(dot+1);
             String srfiNumber = null;
-            if (dot >= 2 || demangled.charAt(0) == ':') {
-                for (int i = 1;  ;  i++) {
+            if (dot > 0) {
+                int numStart = demangled.charAt(0) == ':' ? 1 : 0;
+                for (int i = numStart;  ;  i++) {
                     if (i == dot) {
-                        srfiNumber = demangled.substring(1, dot);
+                        srfiNumber = demangled.substring(numStart, dot);
                         break;
                     }
                     if (Character.digit(demangled.charAt(i), 10) < 0)
@@ -118,41 +122,35 @@ public class ImportFromLibrary extends Syntax
                 }
             }
             if (srfiNumber == null) {
-                tr.error('e', "SRFI library reference must have the form: (srfi :NNN [name])");
+                tr.error('e', "SRFI library reference must have the form: (srfi NNN [name]) or (srfi :NNN [name])");
                 return lname;
             }
             int srfiIndex = SRFI97Map.length;
             for (;;) {
                 if (--srfiIndex < 0) {
-                    tr.error('e', badNameBuffer != null ? badNameBuffer.toString()
-                             : "unknown SRFI number '"+srfiNumber+"' in SRFI library reference");
-                    return lname;
+                    break;
                 }
                 if (!SRFI97Map[srfiIndex][0].equals(srfiNumber))
                     continue;
                 String srfiNameExpected = SRFI97Map[srfiIndex][1];
                 String srfiClass = SRFI97Map[srfiIndex][2];
-                if (srfiName != null && ! srfiName.equals(srfiNameExpected)) {
-                    if (badNameBuffer == null) {
-                        badNameBuffer = new StringBuilder("the name of SRFI ");
-                        badNameBuffer.append(srfiNumber);
-                        badNameBuffer.append(" should be '");
-                    }
-                    else
-                        badNameBuffer.append(" or '");
-                    badNameBuffer.append(srfiNameExpected);
-                    badNameBuffer.append('\'');
-                    continue;
-                }
 
-                if (srfiClass == BUILTIN)
-                    return BUILTIN; // Nothing to do.
-                else if (srfiClass == MISSING) {
-                    tr.error('e', "sorry - Kawa does not support SRFI "+srfiNumber+" ("+srfiNameExpected+')');
+                if (srfiName == null || srfiName.equals(srfiNameExpected))
+                    return srfiClass != MISSING ? srfiClass : lname;
+
+                if (badNameBuffer == null) {
+                    badNameBuffer = new StringBuilder("the name of SRFI ");
+                    badNameBuffer.append(srfiNumber);
+                    badNameBuffer.append(" should be '");
                 }
                 else
-                    lname = srfiClass;
-                break;
+                    badNameBuffer.append(" or '");
+                badNameBuffer.append(srfiNameExpected);
+                badNameBuffer.append('\'');
+            }
+            if (badNameBuffer != null) {
+                tr.error('e', badNameBuffer.toString());
+                return BUILTIN;
             }
         }
         return lname;
@@ -192,9 +190,10 @@ public class ImportFromLibrary extends Syntax
             return;
         }
 
+        String explicitSource = null;
         Object versionSpec = null;
-        String sourcePath = null;
-        StringBuffer sbuf = new StringBuffer();
+        StringBuilder cbuf = new StringBuilder(); // for class name
+        StringBuilder sbuf = new StringBuilder(); // for source file name
         Object libref = pimport;
         while (libref instanceof Pair) {
             Pair pair = (Pair) libref;
@@ -205,47 +204,209 @@ public class ImportFromLibrary extends Syntax
                     tr.error('e', "duplicate version reference - was "+versionSpec);
                 }
                 versionSpec = car;
-                System.err.println("import version "+car);
             } else if (car instanceof String) {
                 if (cdr instanceof Pair)
                     tr.error('e', "source specifier must be last element in library reference");
-                sourcePath = (String) car;
+                explicitSource = (String) car;
             } else {
+                if (cbuf.length() > 0)
+                    cbuf.append('.');
                 if (sbuf.length() > 0)
-                    sbuf.append('.');
-                sbuf.append(Compilation.mangleNameIfNeeded(car.toString()));
+                    sbuf.append('/');
+                String part = car.toString();
+                cbuf.append(Compilation.mangleNameIfNeeded(part));
+                sbuf.append(part);
             }
             libref = cdr;
         }
+        handleImport(sbuf.toString(), explicitSource,
+                     cbuf.toString(),
+                     defs, tr, mapper);
+    }
 
+    /** Do the actual work of importing a module.
+     * @param implicitSource Source name inferred from library name,
+     *   with '/' as separator. Does not include a file extension.
+     * @param explicitSource If non-null, an exlicitly specified
+     *   source file name.
+     */
+    public static void handleImport(String implicitSource, String explicitSource, String requestedClass, ScopeExp defs, Translator tr, require.DeclSetMapper mapper) {
+
+        ModuleManager mmanager = ModuleManager.getInstance();
         ModuleInfo minfo = null;
-        if (sourcePath != null) {
-            minfo = require.lookupModuleFromSourcePath(sourcePath, defs);
-            if (minfo == null) {
-                tr.error('e', "malformed URL: "+sourcePath);
-                return;
-            }
-        }
-        String lname = sbuf.toString();
-        lname = checkSrfi(lname, tr);
+        String lname = checkSrfi(requestedClass, tr);
         if (lname == BUILTIN)
             return; // nothing to do
+        boolean foundSrfi = lname != requestedClass;
 
         int classPrefixPathLength = classPrefixPath.length;
+        Class existingClass = null;
         for (int i = 0;  i < classPrefixPathLength;  i++) {
             String tname = classPrefixPath[i] + lname;
+            minfo = mmanager.searchWithClassName(tname);
+            if (minfo != null)
+                break;
             try {
-                minfo = ModuleManager.getInstance().findWithClassName(tname);
+                existingClass = ObjectType.getContextClass(tname);
+                break;
             } catch (Exception ex) {
-                continue;
             }
         }
-        if (minfo == null) {
-            tr.error('e', "unknown class "+lname);
-            return;
+
+        String currentFileName = tr.getFileName();
+        Path currentSource = currentFileName == null ? null
+            : Path.valueOf(currentFileName).getAbsolute();
+        String currentExtension = currentSource == null ? null
+            : currentSource.getExtension();
+        if (currentExtension == null) {
+            List<String> langExtensions = tr.getLanguage().getExtensions();
+            if (! langExtensions.isEmpty())
+                currentExtension = langExtensions.get(0);
         }
-        require.importDefinitions(null, minfo, mapper,
-                                  tr.formStack, defs, tr);
+
+        boolean hasDot;
+        boolean isAbsolute;
+        if (explicitSource != null) {
+            hasDot = explicitSource.indexOf("./") >= 0;
+            isAbsolute = Path.valueOf(explicitSource).isAbsolute();
+        } else {
+            hasDot = false;
+            isAbsolute = false;
+        }
+
+        ModuleInfo curinfo = tr.getMinfo();
+        String currentClassName = curinfo.getClassName();
+
+        // Is the current module a file - as opposed to (say) a tty?
+        boolean currentIsFile = currentSource instanceof FilePath
+            && ((FilePath) currentSource).toFileRaw().isFile();
+        Path currentRoot = currentIsFile ? currentSource.getDirectory()
+            : Path.currentPath();
+        if (currentIsFile
+            && ! (explicitSource != null && (hasDot || isAbsolute))) {
+            int currentDots = 0;
+            String prefix = currentClassName != null ? currentClassName
+                : tr.classPrefix != null ? tr.classPrefix : "";
+            for (int i = prefix.length(); --i >= 0; )
+                if (prefix.charAt(i) == '.')
+                    currentDots++;
+            if (currentDots > 0) {
+                StringBuilder ups = new StringBuilder("..");
+                for (int i = currentDots; -- i > 0; )
+                    ups.append("/..");
+                currentRoot = currentRoot.resolve(ups.toString());
+            }
+        }
+
+        List<CharSequence> srcSearchPath;
+        boolean skipSourceSearch = minfo != null && explicitSource == null;
+        if (isAbsolute || hasDot || skipSourceSearch) {
+            srcSearchPath = new ArrayList<CharSequence>();
+            if (! skipSourceSearch)
+                srcSearchPath.add(currentRoot.toString());
+        }
+        else
+            srcSearchPath = getImportSearchPath();
+        String pathStr = null;
+        for (CharSequence searchElement : srcSearchPath) {
+            if (isAbsolute)
+                pathStr = explicitSource;
+            else {
+                String pathElement = searchElement.toString();
+                int selectorEnd;
+                int star;
+                int prefixLength = 0;
+                StringBuilder pbuf = new StringBuilder();
+                if (pathElement.length() >= 3
+                    && pathElement.charAt(0) == '<'
+                    && (selectorEnd = pathElement.indexOf('>')+1) > 0) {
+                    StringBuilder prefixBuf = new StringBuilder();
+                    boolean slashNeeded = false;
+                    for (int i = 1; i < selectorEnd-1; i++) {
+                        char ch = pathElement.charAt(i);
+                        if (ch == ' ') {
+                            if (prefixBuf.length() > 0)
+                                slashNeeded = true;
+                        } else {
+                            if (slashNeeded)
+                                prefixBuf.append('/');
+                            prefixBuf.append(ch);
+                            prefixLength += slashNeeded ? 2 : 1;
+                            slashNeeded = false;
+                        }
+                    }
+                    if (! implicitSource.startsWith(prefixBuf.toString()))
+                        continue;
+                    if (implicitSource.length() != prefixLength) {
+                        if (implicitSource.charAt(prefixLength) != '/')
+                            continue;
+                        prefixLength++;
+                    }
+                    star = pathElement.indexOf('*', selectorEnd);
+                    if (star < 0) {
+                         pathElement = pathElement.substring(selectorEnd);
+                    }
+                } else { // No "<...>..." selector
+                    star = pathElement.indexOf('*');
+                    selectorEnd = 0;
+                    if (foundSrfi && explicitSource == null)
+                        continue;
+                }
+
+                if (star >= 0) {
+                    pbuf.append(pathElement.substring(selectorEnd, star));
+                    pbuf.append(implicitSource.substring(prefixLength));
+                    pbuf.append(pathElement.substring(star+1));
+                } else {
+                    pbuf.append(pathElement);
+                    pbuf.append('/');
+                    if (explicitSource != null)
+                        pbuf.append(explicitSource);
+                    else {
+                        pbuf.append(implicitSource);
+                        if (currentExtension != null) {
+                            pbuf.append('.');
+                            pbuf.append(currentExtension);
+                        }
+                    }
+                }
+                pathStr = pbuf.toString();
+            }
+            Path path = currentRoot.resolve(pathStr);
+            // Might be more efficient to first check the ModuleManager,
+            // before asking the file-system.  FIXME
+            long lastModifiedTime = path.getLastModified();
+            if (lastModifiedTime != 0) {
+                if (minfo != null) {
+                    String pstring = path.getCanonical().toString();
+                    Path infoPath = minfo.getSourceAbsPath();
+                    if (infoPath == null
+                        || ! (pstring.equals(infoPath.toString()))) {
+                        tr.error('w', "ignoring source file at "+pstring
+                                 +" - instead using class "+minfo.getClassName()
+                                 +(infoPath==null?""
+                                   :(" from "+infoPath.toString())));
+                    }
+                } else
+                    minfo = mmanager.findWithSourcePath(path, pathStr);
+                // Should save lastModifiedTime in minfo FIXME
+                if (foundSrfi)
+                    lname = requestedClass;
+                break;
+            }
+        }
+
+        if (existingClass != null) {
+            if (minfo == null)
+                minfo = mmanager.findWithClass(existingClass);
+            else
+                minfo.setModuleClass(existingClass);
+        }
+        if (minfo == null)
+            tr.error('e', "unknown library ("+implicitSource.replace('/', ' ')+")");
+        else
+            require.importDefinitions(lname, minfo, mapper,
+                                      tr.formStack, defs, tr);
     }
 
     public Expression rewriteForm(Pair form, Translator tr) {
@@ -382,6 +543,13 @@ public class ImportFromLibrary extends Syntax
         return null;
     }
 
+    public static final ThreadLocal<List<CharSequence>> searchPath
+        = new InheritableThreadLocal<List<CharSequence>>();
+
+    public static List<CharSequence> getImportSearchPath() {
+        return Include.getSearchPath(searchPath, "kawa.import.path", ".");
+    }
+
     public static final SimpleSymbol exceptSymbol = Symbol.valueOf("except");
     public static final SimpleSymbol librarySymbol = Symbol.valueOf("library");
     public static final SimpleSymbol onlySymbol = Symbol.valueOf("only");
