@@ -152,8 +152,6 @@ public class Scheme extends LispLanguage
   {
       environ = nullEnvironment;
 
-      environ.addLocation(LispLanguage.lookup_sym, null, getNamedPartLocation);
-
       defSntxStFld("lambda", "kawa.standard.SchemeCompilation", "lambda");
       defSntxStFld("$bracket-apply$", "gnu.kawa.lispexpr.BracketApply", "instance");
       defSntxStFld("$string$", "kawa.lib.syntax");
@@ -702,8 +700,6 @@ public class Scheme extends LispLanguage
                    "define_macro");
       defSntxStFld("define-syntax-case", "kawa.lib.syntax");
       defSntxStFld("syntax-case", "kawa.standard.syntax_case", "syntax_case");
-      defSntxStFld("%define-syntax", "kawa.standard.define_syntax",
-                   "define_syntax");
       defSntxStFld("define-rewrite-syntax", "kawa.standard.define_syntax",
                    "define_rewrite_syntax");
       defSntxStFld("syntax", "kawa.standard.syntax", "syntax");
