@@ -111,6 +111,7 @@ public class require extends Syntax
         map("android-defs", "gnu.kawa.android.defs");
         map("javafx-defs", "gnu.kawa.javafx.defs");
         map("syntax-utils", SLIB_PREFIX + "syntaxutils");
+        map("quaternions", "kawa.lib.kawa.quaternions");
     }
 
     public static String mapFeature(String featureName) {
@@ -160,25 +161,22 @@ public class require extends Syntax
             }
             return importDefinitions(null, info, null, tr.formStack, defs, tr);
         } else if (name instanceof Symbol && ! tr.selfEvaluatingSymbol(name)) {
-            type = tr.getLanguage().getTypeFor(tr.rewrite(name, false));
-            String cname = null;
-            if (type instanceof ClassType)
-                cname = type.getName();
-            else if (type == null && name instanceof SimpleSymbol)
-                cname = ((SimpleSymbol) name).getName();
-            if (cname != null && args.getCdr() instanceof Pair) {
+            String requestedClass = name.toString();
+            int nlen = requestedClass.length();
+            if (nlen > 2 && requestedClass.charAt(0) == '<'
+                && requestedClass.charAt(nlen-1) == '>')
+                requestedClass = requestedClass.substring(1, nlen-1);
+            String implicitSource = requestedClass.replace('.', '/');
+            String explicitSource = null;
+            if (args.getCdr() instanceof Pair) {
                 Object sname = ((Pair) args.getCdr()).getCar();
                 if (sname instanceof CharSequence) {
-                    String sourceName = sname.toString();
-                    ModuleInfo info = lookupModuleFromSourcePath(sourceName, defs);
-                    if (info == null) {
-                        tr.error('e', "malformed URL: "+sourceName);
-                        return false;
-                    }
-                    return importDefinitions(cname, info, null,
-                                             tr.formStack, defs, tr);
-                }
+                    explicitSource = sname.toString();
+                } // else ERROR
             }
+            ImportFromLibrary.handleImport(implicitSource, explicitSource, requestedClass, defs, tr, null);
+            return true;
+
         }
         if (! (type instanceof ClassType)) {
             if (type != null)
@@ -214,9 +212,9 @@ public class require extends Syntax
      *   or null if unknown.
      */
     public static boolean
-        importDefinitions(String className, ModuleInfo info,
-                           DeclSetMapper mapper, FormStack forms, 
-                           ScopeExp defs, Compilation tr) {
+    importDefinitions(String className, ModuleInfo info,
+                      DeclSetMapper mapper, FormStack forms, 
+                      ScopeExp defs, Compilation tr) {
         ModuleManager manager = ModuleManager.getInstance();
         long now;
         if ((info.getState() & 1) == 0
@@ -228,7 +226,6 @@ public class require extends Syntax
             try {
                 InPort fstream = InPort.openFile(info.getSourceAbsPath());
                 info.clearClass();
-                info.setClassName(className);
                 int options = Language.PARSE_PROLOG;
                 if (tr.immediate)
                     options |= Language.PARSE_IMMEDIATE;
@@ -245,7 +242,34 @@ public class require extends Syntax
                 // otherwise ignore it - it's already been recorded in messages.
                 return false;
             }
-            info.setClassName(comp.getModule().classFor(comp).getName());
+            String compiledClassName = comp.getModule().classFor(comp).getName();
+ 
+            if (className != null) {
+                Map<String,ModuleInfo> subModuleMap = comp.subModuleMap;
+                ModuleInfo modinfo;
+                if (subModuleMap != null) {
+                    modinfo = subModuleMap.get(className);
+                } else
+                    modinfo = null;
+                if (modinfo == null) {
+                    String[] classPrefixPath
+                        = ImportFromLibrary.classPrefixPath;
+                    int classPrefixPathLength = classPrefixPath.length;
+                    for (int i = 0;  i < classPrefixPathLength;  i++) {
+                        String tname = classPrefixPath[i] + className;
+                        if (tname.equals(compiledClassName)) {
+                            modinfo = info;
+                            break;
+                        }
+                    }
+                }
+                if (modinfo == null)
+                    tr.error('e', ("file '"+info.getSourceAbsPath()
+                                   +"' does not declare library '"
+                                   +className+"'"));
+                else
+                    info = modinfo;
+            }
         }
 
         ModuleInfo curinfo = tr.getMinfo();
@@ -278,35 +302,13 @@ public class require extends Syntax
 
         Map<Symbol,Declaration> dmap
             = new LinkedHashMap<Symbol,Declaration>();
+        Map<String,Declaration> moduleReferences = null;
+       
         for (Declaration fdecl = mod.firstDecl();
              fdecl != null;  fdecl = fdecl.nextDecl()) {
             if (fdecl.isPrivate())
                 continue;
 
-            Symbol aname = (Symbol) fdecl.getSymbol();
-            boolean isStatic = fdecl.getFlag(Declaration.STATIC_SPECIFIED);
-            if (! isStatic && decl == null) {
-                String iname = tname.replace('.', '$') + "$instance";
-                decl = new Declaration(SimpleSymbol.valueOf(iname), type);
-                decl.setPrivate(true);
-                decl.setFlag(Declaration.IS_CONSTANT
-                             |Declaration.MODULE_REFERENCE);
-                defs.addDeclaration(decl);
-
-                decl.noteValue(dofind);
-                SetExp sexp = new SetExp(decl, dofind);
-                sexp.setLine(tr);
-                sexp.setDefining(true);
-                forms.push(sexp);
-                decl.setFlag(Declaration.EARLY_INIT);
-                // If Runnable, we need to set decl value in initializer,
-                // and later 'run' it, so it needs to be stored in a field.
-                if (isRunnable)
-                    decl.setSimple(false);
-
-                decl.setFlag(Declaration.TYPE_SPECIFIED);
-            }
-
             if (fdecl.field != null) {
                 String fname = fdecl.field.getName();
                 if (fname.equals("$instance"))
@@ -318,19 +320,11 @@ public class require extends Syntax
 
             if (fdecl.field != null
                 && fdecl.field.getName().endsWith("$instance")) {
-                Declaration old = defs.lookup(aname, language, language.getNamespaceOf(fdecl));
-                if (old != null)
-                    continue;
-                Declaration adecl = defs.addDeclaration(aname);
-                adecl.setFlag(Declaration.IS_CONSTANT
-                              |Declaration.TYPE_SPECIFIED
-                              |Declaration.MODULE_REFERENCE);
-                adecl.setType(fdecl.getType());
-                ReferenceExp fref = new ReferenceExp(fdecl);
-                fref.setContextDecl(decl);
-                linkDecls(adecl, fdecl, fref, forms, tr);
+                if (moduleReferences == null)
+                    moduleReferences = new HashMap<String,Declaration>();
+                moduleReferences.put(fdecl.field.getName(), fdecl);
             } else
-                dmap.put(aname, fdecl);
+                dmap.put((Symbol) fdecl.getSymbol(), fdecl);
         }
 
         if (mapper != null)
@@ -354,19 +348,42 @@ public class require extends Syntax
                      && (Declaration.followAliases(old)
                          == Declaration.followAliases(fdecl)))
                 continue;
-            else {
-                if (old != null
-                    && (old.getFlag(Declaration.NOT_DEFINING | Declaration.IS_UNKNOWN))) {
-                    old.setFlag(false, Declaration.NOT_DEFINING|Declaration.IS_UNKNOWN);
-                    adecl = old;
-                } else {
-                    adecl = defs.addDeclaration(aname);
-                    if (old != null)
-                        ScopeExp.duplicateDeclarationError(old, adecl, tr);
-                }
-                adecl.setAlias(true);
-                adecl.setIndirectBinding(true);
+
+            if (decl == null && ! fdecl.getFlag(Declaration.STATIC_SPECIFIED)) {
+                String iname = tname.replace('.', '$') + "$instance";
+                decl = new Declaration(SimpleSymbol.valueOf(iname), type);
+                decl.setPrivate(true);
+                decl.setFlag(Declaration.IS_CONSTANT
+                             |Declaration.MODULE_REFERENCE);
+                defs.addDeclaration(decl);
+
+                decl.noteValue(dofind);
+                SetExp sexp = new SetExp(decl, dofind);
+                sexp.setLine(tr);
+                sexp.setDefining(true);
+                forms.push(sexp);
+                decl.setFlag(Declaration.EARLY_INIT);
+                // If Runnable, we need to set decl value in initializer,
+                // and later 'run' it, so it needs to be stored in a field.
+                if (isRunnable)
+                    decl.setSimple(false);
+
+                decl.setFlag(Declaration.TYPE_SPECIFIED);
             }
+
+            if (old != null
+                && (old.getFlag(Declaration.NOT_DEFINING | Declaration.IS_UNKNOWN))) {
+                old.setFlag(false, Declaration.NOT_DEFINING|Declaration.IS_UNKNOWN);
+                adecl = old;
+            } else {
+                adecl = defs.addDeclaration(aname);
+                if (old != null)
+                    ScopeExp.duplicateDeclarationError(old, adecl, tr);
+            }
+
+            adecl.setAlias(true);
+            adecl.setIndirectBinding(true);
+
             ReferenceExp fref = new ReferenceExp(fdecl);
             fref.setContextDecl(decl);
             fref.setDontDereference(true);
@@ -380,14 +397,28 @@ public class require extends Syntax
                 Declaration xdecl = ((ReferenceExp) fval).getBinding();
                 aref.setBinding(xdecl);
                 // xdecl can be null on an error.
-                if (xdecl != null && xdecl.needsContext())
-                {
+                if (xdecl != null && xdecl.needsContext()) {
                     String iname
                         = (xdecl.field.getDeclaringClass().getName().replace('.', '$')
                            + "$instance");
-                    Declaration cdecl = defs.lookup(SimpleSymbol.valueOf(iname));
+                    Declaration cdecl = moduleReferences == null ? null
+                        : moduleReferences.get(iname);
+                    if (cdecl != null) {
+                        if (cdecl.context != defs) {
+                        Declaration acdecl = defs.addDeclaration(SimpleSymbol.valueOf(iname));
+                        moduleReferences.put(iname, acdecl);
+                        acdecl.setFlag(Declaration.IS_CONSTANT
+                                      |Declaration.TYPE_SPECIFIED
+                                      |Declaration.MODULE_REFERENCE);
+                        acdecl.setType(cdecl.getType());
+                        ReferenceExp cref = new ReferenceExp(cdecl);
+                        cref.setContextDecl(decl);
+                        linkDecls(acdecl, cdecl, cref, forms, tr);
+                        cdecl = acdecl;
+                    }
                     cdecl.setFlag(Declaration.EXPORT_SPECIFIED);
                     aref.setContextDecl(cdecl);
+                    }
                 }
             }
         }
@@ -430,7 +461,6 @@ public class require extends Syntax
         adecl.noteValue(fref);
         adecl.setFlag(Declaration.IS_IMPORTED);
         tr.push(adecl);  // Add to translation env.
-
     }
 
     public Expression rewriteForm(Pair form, Translator tr) {
