@@ -1,4 +1,4 @@
-(test-init "Objects" 138)
+(test-init "Objects" 143)
 
 ;; Force procedure to be applied without being inlined:
 (define-syntax force-eval
@@ -159,8 +159,11 @@
 
 (define-variable internal-node-name list)
 (require <module2>)
+(test "fun2" fun1)
+(test "fun1" 'fun2 (fun2))
+(test '("fun1" "fun2") fun1fun2)
 (test 4 list-length-1 '(a b c d))
-(test 2 list-length-3 '(a b))
+(test 2 list-length-5 '(a b))
 (test 0 length (classify))
 
 (test 3 length-diff1 "abcdef" "abc")
@@ -180,6 +183,14 @@
 (define-variable dvar2 3)
 (require <module3>)
 
+(test 13 'Savannah-bug-40822 (macro2))
+
+(let* ((all0 (all-zeros))
+       (nlen (gnu.lists.LList:listLength all0 #t)))
+  (test "#0=(0 . #0#) len:-1" 'Savannah-bug-43233
+        (format #f "~w len:~d" all0 nlen)))
+
+
 (define-variable dvar3 4)
 (test '(2 3 13) 'dvar-test-1 dvar-test-1)
 (set! dvar1 1)
